package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.Sequence;

/**
 * An interface to represent a data column.
 * <p>
 * <i>Implementation note: Will be backed up by a column in an SQL table.</i> 
 */
public interface DataColumn {
    public int getTextAreaHeight();
    /**
     * Returns the Java type of the column.
     * @return the Java type of the column.
     */
    public Class getType();

    /**
     * Returns <code>true</code> iff updates can only be performed when
     * this column is specified.
     * @return wether or not this column is required to perform an update.
     * @see DataManager#update(Record)
     */
    public boolean isRequiredForUpdate();

    /**
     * Returns the name of this column.
     * @return the name of this column
     */
    public String getName();

    /**
     * Returns the associated {@link DataScheme} or <code>null</code> if there
     * is none. The latter is the general case.
     * <p>
     * <strong>Example:</strong> This column represents a customer id, and it
     * is part of a {@link DataScheme} that represents repair orders. Then
     * this method returns an <storng>other</strong> {@link DataScheme} 
     * which represents a customer.
     * @return the associated {@link DataScheme} or <code>null</code> if there
     * is none.
     */
    public DataScheme getAssociatedDataScheme();

    /**
     * Returns the name in the associated {@link DataScheme}.
     * <p>
     * <strong>Example:</strong> customerid
     * @return the name in the associated {@link DataScheme}; <code>null</code>
     * iff {@link #getAssociatedDataScheme()} returns <code>null</code>.
     */
    public String getNameInAssociatedDataScheme();

    public Sequence<String> getChoices();

    public boolean isHidden();

    public boolean isSmall();
    
    public boolean isIndirectlyHidden();
}
