package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.Sequence;

import java.io.IOException;

/**
 * Provides an interface to access the underlying database.
 */ 
public interface DataManager {
    /**
     * Returns all {@link Record records} that match the {@link Condition}.
     * @param condition the matching {@link Condition}
     * @return all matching {@link Record records}
     * @throws IOException iff an I/O error occurs
     */ 
    public Sequence<Record> find(Condition condition) throws IOException;

    /**
     * Updates the provided {@link Record} with its changed fields.
     * The record is identified by its ID.
     * @param record the record with the new values
     * @throws IOException iff an I/O error occurs
     */ 
    public void update(Record record) throws IOException;

    /**
     * Inserts a new {@link Record}.
     * @param record the new record
     * @throws IOException iff an I/O error occurs
     */ 
    public void insert(Record record) throws IOException;
    
    /**
     * Deletes the {@link Record} with the specified ID.
     * @param recordNumber the ID of the record
     * @throws IOException iff an I/O error occurs
     */ 
    public void delete(long recordNumber) throws IOException;
}
