package org.kaiwitte.workbench.data;

import java.util.HashMap;
import java.util.Map;
import java.sql.SQLException;
import java.io.IOException;

/**
 * Default implementation of {@link DataManagerFactory}.
 */ 
public class DataManagerFactoryImpl implements DataManagerFactory {
    private Map<DataScheme,DataManager> map =
        new HashMap<DataScheme,DataManager>();
    private String driver;
    private String database;

    /**
     * Creates a new instance.
     * @param driver the driver as a fully qualified class name
     * @param database the database URL as specified by JDBC
     */ 
    public DataManagerFactoryImpl(String driver, String database) {
        this.driver = driver;
        this.database = database;
    }

    public DataManager getDataManager(DataScheme scheme) 
            throws IOException {
        if (map.get(scheme) == null) {
            DataManager manager = null;
            try {
                manager = new DataManagerImpl(driver, database, scheme);
            } catch (SQLException e) {
                throw new IOException(e.getMessage());
            } catch (ClassNotFoundException e) {
                throw new IOException(e.getMessage());
            } catch (IllegalAccessException e) {
                throw new IOException(e.getMessage());
            } catch (InstantiationException e) {
                throw new IOException(e.getMessage());
            }
            map.put(scheme, manager);
        }
        return map.get(scheme);
    }

}
