package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.Sequence;
import java.util.Iterator;

public final class DataSchemaFactoryCreator {
    private DataSchemaFactoryCreator() {
        throw new UnsupportedOperationException();
    }
    
    public static BackupDataSchemeFactory createBackupSchemaFactory() {
        return new BackupDataSchemeFactory() {
            public DataScheme createDataScheme(DataScheme in) {
                return new BackupDataScheme(in);
            }
        };
    }
    
    private static final class BackupDataScheme implements DataScheme {
        private final DataScheme scheme;
        private static final String BACKUP_SUFFIX = "_backup";
        private DataScheme backupScheme;

        public BackupDataScheme(DataScheme scheme) {
            this.scheme = scheme;
        }

        public int getLength() {
            return scheme.getLength();
        }

        public Sequence<DataColumn> getColumns() {
            return scheme.getColumns();
        }

        public String getName() {
            return scheme.getName() + BACKUP_SUFFIX;
        }

        public String getMailCondition() {
            return scheme.getMailCondition();
        }

        public DataScheme getBackupScheme() {
            return backupScheme;
        }

        public String getAutoLoadCondition() {
            return scheme.getAutoLoadCondition();
        }

        public Iterator<String[]> printTemplates() {
            return scheme.printTemplates();
        }
    }
    
}
