package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.Sequence;

import java.sql.SQLException;
import java.util.Iterator;

/**
 * Represents a data scheme.
 * <p>
 * <i>Implementation note: Will be backed up by a table in an SQL database.</i> 
 */ 
public interface DataScheme {
    /**
     * Returns the number of {@link DataColumn DataColumns}.
     * @return the number of {@link DataColumn DataColumns}
     */ 
    public int getLength();
    
    /**
     * Returns the {@link DataColumn DataColumns}.
     * @return the {@link DataColumn DataColumns}
     */ 
    public Sequence<DataColumn> getColumns();
    
    /**
     * Returns the name of this column.
     * @return the name of this column
     */ 
    public String getName();
    
    /**
     * Returns the condition for records that are build according to this
     * <code>DataScheme</code> 
     * when a mail is to be sent. String types
     * are NOT quoted. E. g.: <code>status=erledigt</code>.
     * @return condition when a mail is to be sent
     */ 
    public String getMailCondition();
    
    public DataScheme getBackupScheme();

    /**
     * Returns the condition for records that are build according to this
     * <code>DataScheme</code> 
     * when they shall be automatically loaded to a visible table on
     * program start. Strict SQL syntax. String types
     * ARE quoted. E. g.: 
     * <ul>
     * <li>status='nicht erledigt'</li>
     * <li>age&lt;27</li>
     * <li>status='nicht erledigt' AND age&lt;27</li>
     * @return condition for when the record requires special attention
     */ 
    public String getAutoLoadCondition();
    
    public Iterator<String[]> printTemplates();
}
