package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.Sequence;

import java.io.IOException;
import java.sql.SQLException;

/**
 * Provides an interface to access related {@link DataManager}s.
 */ 
public interface MetaDataManager {
    /**
     * Returns all {@link MetaRecord}s that match the {@link Condition}.
     * @param condition the matching {@link Condition}
     * @return all matching {@link MetaRecord MetaRecords}
     * @throws IOException iff an I/O error occurs
     */ 
    public Sequence<MetaRecord> find(Condition condition) throws IOException;

    /**
     * Establishes a connection.
     * @throws IOException iff an I/O error occurs
     */
    public void connect() throws IOException;
    
    /**
     * Returns true iff the connection appears to be open.
     * @throws IOException iff an I/O error occurs
     */
    public boolean isConnected() throws IOException;

    /**
     * Updates the provided {@link MetaRecord} with its changed fields.
     * The record is identified by its ID. Connected 
     * {@link Record records} are updated if they already have an ID.
     * Otherwise they are created.
     * @param record the record with the new values
     * @throws IOException iff an I/O error occurs
     */ 
    public void update(MetaRecord record) throws IOException;

    /**
     * Inserts a new {@link MetaRecord}.
     * @param record the new <code>MetaRecord</code>
     * @return the <code>MetaRecord</code> from the parameter with the
     * proper ID set
     * @throws IOException iff an I/O error occurs
     */ 
    public MetaRecord insert(MetaRecord record) throws IOException;

    /**
     * Deletes the {@link MetaRecord} with the specified ID.
     * @param recordNumber the ID of the record
     * @throws IOException iff an I/O error occurs
     */ 
    public void delete(long recordNumber) throws IOException;
}
