package org.kaiwitte.workbench.data;

import java.util.HashMap;
import java.util.Map;
import java.sql.SQLException;
import java.io.IOException;

/**
 * Default implementation of {@link MetaDataManagerFactory}.
 */ 
public class MetaDataManagerFactoryImpl implements MetaDataManagerFactory {
    private Map<MetaScheme,MetaDataManager> map =
        new HashMap<MetaScheme,MetaDataManager>();
    private String driver;
    private String database;
    
    /**
     * Creates a new instance.
     * @param driver the driver as a fully qualified class name
     * @param database the database URL as specified by JDBC
     */ 
    public MetaDataManagerFactoryImpl(String driver, String database) {
        this.driver = driver;
        this.database = database;
    }

    public MetaDataManager getMetaDataManager(MetaScheme scheme) 
            throws IOException {
        if (map.get(scheme) == null) {
            MetaDataManager manager = 
                null;
            try {
                manager = new MetaDataManagerImpl(driver, database, scheme);
            } catch (ClassNotFoundException e) {
                throw new IOException(e.getMessage());
            } catch (IllegalAccessException e) {
                throw new IOException(e.getMessage());
            } catch (InstantiationException e) {
                throw new IOException(e.getMessage());
            }
            map.put(scheme, manager);
        }
        return map.get(scheme);
    }

    public boolean isAllConnected() {
        boolean ret = true;
        for (MetaDataManager manager : map.values()) {
            try {
                ret &= manager.isConnected();
            } catch (IOException e) {
                return false;
            }
        }
        return ret;
    }

    public void reconnectAll() throws IOException {
        for (MetaDataManager manager : map.values()) {
            manager.connect();
        }
    }
}
