package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.Sequence;
import net.tmorris.adt.sequence.ArraySequenceImpl;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

/**
 * Default implementation of a {@link MetaRecord}.
 */ 
public class MetaRecordImpl implements MetaRecord {
    private final List values = new ArrayList();
    private final Record[] records;
    private Map<DataScheme,Record> recordToScheme =
        new HashMap<DataScheme,Record>();
    private final List<String> columnNames = new ArrayList<String>();

    /**
     * Creates a new instance.
     * @param records the records to be wrapped
     */ 
    public MetaRecordImpl(Record[] records) {
        this.records = records.clone();
        for (Record r : records) {
            recordToScheme.put(r.getScheme(), r);
        }
        addValues(records[0]);
    }
    
    /**
     * Creates a new instance.
     * @param values the values for this record
     */ 
    public MetaRecordImpl(MetaScheme metaScheme, Object... values) {
        int i = 0;
        List<Record> records = new ArrayList<Record>();
        for (DataScheme scheme : metaScheme.getDataSchemes()) {
            List recordValues = new ArrayList();
            for (DataColumn column : scheme.getColumns()) {
                recordValues.add(values[i++]);
            }
            Record r = new MutableRecordImpl(scheme, recordValues.toArray());
            records.add(r);
        }
        this.records = records.toArray(new Record[0]);
        for (Record r : records) {
            recordToScheme.put(r.getScheme(), r);
        }
        addValues(records.get(0));
    }

    private void addValues(Record record) {
        DataScheme scheme = record.getScheme();
        for (int i = 0; i < scheme.getColumns().getSize(); i++) {
            DataScheme associatedDataScheme = 
                scheme.getColumns().get(i).getAssociatedDataScheme();
            if (associatedDataScheme != null) {
                Record associatedRecord = recordToScheme.get(
                                        associatedDataScheme);
                if (associatedRecord == null) {
                    throw new IllegalArgumentException(
                        "missing associated record for record " + record
                    );
                }
                addValues(
                    associatedRecord);
            }
            else {
                values.add(record.getValue(i));
                columnNames.add(scheme.getColumns().get(i).getName());
            }
        }
    }

    public Object getValue(int column) {
        return values.get(column);
    }

    public String getColumnName(int column) {
        return columnNames.get(column);
    }

    public Sequence<Record> getRecords() {
        return ArraySequenceImpl.getArraySequence()
                .getSequence(records.clone());
    }

    @Override 
    public String toString() {
        StringBuilder ret = new StringBuilder();
        for (int i = 0; i < records.length; i++) {
            Record record = records[i];
            ret.append(record.getID() + ": {");
            for (int j = 0; j < record.getScheme().getLength(); j++) {
                ret.append(record.getValue(j));
                ret.append(",");
            }
            ret.append("} ");
        }
        return ret.toString();
    }
}
