package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.ArraySequenceImpl;
import net.tmorris.adt.sequence.Sequence;

import static java.lang.System.*;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

public class MetaSchemeFactory {
    private static final Map<DataScheme,MetaScheme> schemes =
        new HashMap<DataScheme, MetaScheme>();

    private MetaSchemeFactory() {
        throw new UnsupportedOperationException();
    }

    public static synchronized MetaScheme getMetaScheme(DataScheme scheme) {
        MetaScheme ret = schemes.get(scheme);
        if (ret == null) {
            ret = new MetaSchemeImpl(scheme);
            schemes.put(scheme, ret);
        }
        return ret;
    }

    /**
     * A default implementation of a {@link MetaScheme}.
     */
    private static final class MetaSchemeImpl implements MetaScheme {
        List<DataColumn> dataColumns =
            new ArrayList<DataColumn>();
        List<DataScheme> dataSchemes =
            new ArrayList<DataScheme>();
        private String name;

        /**
         * Creates a new instance.
         * @param scheme the primary scheme. The related schemes, if any,
         * are automatically retrieved by calling 
         * {@link DataColumn#getAssociatedDataScheme()} on the passed
         * scheme's {@link DataColumn}s. 
         */
        public MetaSchemeImpl(DataScheme scheme) {
            addColumns(scheme);
            this.name = scheme.getName();
        }

        private void addColumns(DataScheme scheme) {
            dataSchemes.add(scheme);
            for (DataColumn column : scheme.getColumns()) {
                if (column.getAssociatedDataScheme() != null) {
                    addColumns(column.getAssociatedDataScheme());
                }
                else {
                    dataColumns.add(column);
                }
            }
        }

        public int getLength() {
            return dataColumns.size();
        }

        public Sequence<DataColumn> getColumns() {
            return ArraySequenceImpl.getArraySequence()
                .getSequence(dataColumns.toArray(new DataColumn[0]));
        }

        public String getName() {
            return name;
        }

        public String toString() {
            StringBuilder ret = new StringBuilder();
            for (DataColumn d : dataColumns) {
                ret.append(d);
                ret.append(" | ");
            }
            return ret.toString();
        }

        public Sequence<DataScheme> getDataSchemes() {
            return ArraySequenceImpl.getArraySequence()
                .getSequence(dataSchemes.toArray(new DataScheme[0]));
        }
    }
}
