package org.kaiwitte.workbench.data;

import net.tmorris.adt.sequence.Sequence;

import static java.lang.System.*;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * A <code>DataManager</code> that encloses multiple other DataManagers,
 * making backup copies of new entries.
 */
class MultiDataManager implements DataManager {
    private final List<DataManager> managers;

    public MultiDataManager(DataManager... managers) {
        this.managers = Arrays.asList(managers.clone());
    }

    public Sequence<Record> find(Condition condition) throws IOException {
        return managers.get(0).find(condition);
    }

    public void update(Record record) throws IOException {
        managers.get(0).update(record);
    }

    public void insert(Record record) throws IOException {
        for (DataManager manager : managers) {
            manager.insert(record);
        }
    }

    public void delete(long recordNumber) throws IOException {
        managers.get(0).delete(recordNumber);
    }
}
