package org.kaiwitte.workbench.data;

import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * Default implementation of a {@link MutableRecord}.
 */ 
public class MutableRecordImpl implements MutableRecord {
    private long id;
    private List<Object> fields = new ArrayList<Object>();
    private final DataScheme scheme;

    /**
     * Creates a new instance.
     * @param id the id of the record
     * @param scheme the data scheme that describes this record's structure
     * @param fields the content
     */ 
    public MutableRecordImpl(long id, DataScheme scheme, Object... fields) {
        this.scheme = scheme;
        this.id = id;
        this.fields.addAll(Arrays.asList(fields));
    }

    /**
     * Creates a new instance.
     * @param scheme the data scheme that describes this record's structure
     * @param fields the content
     */ 
    public MutableRecordImpl(DataScheme scheme, Object... fields) {
        this.scheme = scheme;
        //@todo make copy to encapsulate
        this.fields.addAll(Arrays.asList(fields));
    }
    
    public void setValue(int fieldNumber, Object fieldValue) {
        fields.set(fieldNumber, fieldValue);
    }

    public void setID(long id) {
        this.id = id;
    }

    public long getID() {
        return id;
    }

    public Object getValue(int fieldNumber) {
        return fields.get(fieldNumber);
    }

    public DataScheme getScheme() {
        return scheme;
    }

    @Override 
    public int hashCode() {
        int ret = fields.size() == 0 ? 0 : fields.get(0).hashCode();
        for (int i = 1; i < fields.size(); i++) {
            ret ^= fields.get(i).hashCode();
        }
        ret &= id << 32;
        ret ^= id >> 32;
        return ret;
    }

    @Override 
    public boolean equals(Object obj) {
        if (obj == null || !(obj instanceof Record)) {
            return false;
        }

        Record comp = (Record) obj;
        boolean ret = true;

        for (int i = 0; i < fields.size(); i++) {
            ret &= comp.getValue(i).equals(fields.get(i));
        }

        ret &= comp.getID() == id;

        return ret;
    }

    @Override 
    public String toString() {
        return id + fields.toString();
    }
}
