package org.kaiwitte.workbench.mail;

import static java.lang.System.*;
import org.kaiwitte.workbench.data.MetaRecord;
import org.kaiwitte.workbench.data.Record;
import org.kaiwitte.workbench.data.DataScheme;
import org.kaiwitte.workbench.data.DataColumn;
import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.tools.Tools;

import javax.mail.Session;
import javax.mail.Message;
import javax.mail.Transport;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.InternetAddress;
import java.io.IOException;
import java.util.Properties;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;

public class MailCreatorImpl implements MailCreator {
    public String[][] createMail(MetaRecord metaRecord, UIConfig config,
                                 String... templates) 
            throws IOException {
        List<String[]> mails = new ArrayList<String[]>();
        
        for (String resource : templates) {
            String content = Tools.readFile(resource);
            content = Tools.replaceVariables(metaRecord, content, config);
            
            String from = config.getOption("from");
            String to = null;
            String host = config.getOption("smtphost");
            
            int field = 0;
            for (Record r : metaRecord.getRecords()) {
                DataScheme scheme = r.getScheme();
                for (DataColumn column : scheme.getColumns()) {
                    if (metaRecord.getColumnName(field).equals("email")) {
                        to = metaRecord.getValue(field).toString();
                    }
                    if (column.getAssociatedDataScheme() == null) {
                        field++;
                    }
                }
            }
            
            if (to == null) {
                throw new IOException(
                    "field 'email' not found in scheme for record "
                    + metaRecord);
            }

            String mailTemplate = Tools.readFile(resource);
                
            String mailBody = Tools.replaceVariables(metaRecord, 
                                                     mailTemplate,
                                                     config);

            mails.add(
                new String[] {from, to, mailBody}
            );
        }
        
        return mails.toArray(new String[0][0]);
    }
}
