package org.kaiwitte.workbench.mail;

import static java.lang.System.*;
import org.kaiwitte.workbench.data.MetaRecord;
import org.kaiwitte.workbench.data.Record;
import org.kaiwitte.workbench.data.DataScheme;
import org.kaiwitte.workbench.data.DataColumn;
import org.kaiwitte.workbench.tools.Tools;
import org.kaiwitte.workbench.printing.DocumentRenderer;
import org.kaiwitte.workbench.ui.UIConfig;

import javax.swing.*;
import javax.mail.Session;
import javax.mail.Message;
import javax.mail.Transport;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.InternetAddress;
import java.io.IOException;
import java.util.Properties;
import java.util.Date;

public class RecordMailerImpl implements RecordMailer {
    public void mailRecord(MetaRecord metaRecord, UIConfig config,
                           String... templates)
            throws IOException {
        for (String resource : templates) {
            String content = Tools.readResource(resource);
            content = Tools.replaceVariables(metaRecord, content, config);
            
            String from = config.getOption("from");
            String to = null;
            String host = config.getOption("smtphost");
            
            int field = 0;
            for (Record r : metaRecord.getRecords()) {
                DataScheme scheme = r.getScheme();
                for (DataColumn column : scheme.getColumns()) {
                    if (metaRecord.getColumnName(field).equals("email")) {
                        to = metaRecord.getValue(field).toString();
                    }
                    if (column.getAssociatedDataScheme() == null) {
                        field++;
                    }
                }
            }
            
            if (to == null) {
                throw new IOException(
                    "field 'email' not found in scheme for record "
                    + metaRecord);
            }

            boolean debug = false;
            // create some properties and get the default Session
            Properties props = new Properties();
            props.put("mail.smtp.host", host);
            props.put("mail.debug", Boolean.toString(debug));

            Session session = Session.getInstance(props, null);
            session.setDebug(debug);
            
            // create a message
            Message msg = new MimeMessage(session);
            try {
                msg.setFrom(new InternetAddress(from));
                InternetAddress[] address = {new InternetAddress(to)};
                msg.setRecipients(Message.RecipientType.TO, address);
                msg.setSubject(config.getOption("subject"));
                msg.setSentDate(new Date());

                // If the desired charset is known, you can use
                // setText(text, charset)
                
                String mailTemplate = Tools.readResource(resource);
                
                String mailBody = Tools.replaceVariables(metaRecord, 
                                                         mailTemplate,
                                                         config);
                
                msg.setText(mailBody);
                
                Transport.send(msg);
            } catch (MessagingException e) {
                throw new IOException(e.getMessage());
            }
        }
    }
}
