package org.kaiwitte.workbench.printing;

import org.kaiwitte.workbench.data.MetaRecord;
import org.kaiwitte.workbench.data.Record;
import org.kaiwitte.workbench.data.DataScheme;
import org.kaiwitte.workbench.data.DataColumn;
import org.kaiwitte.workbench.tools.Tools;
import org.kaiwitte.workbench.ui.UIConfig;
import static java.lang.System.*;

import javax.swing.text.html.HTMLDocument;
import javax.swing.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.regex.Pattern;

import net.tmorris.adt.sequence.Sequence;

/**
 * The default implementation of a <code>RecordPrinter</code>.
 */
public class RecordPrinterImpl implements RecordPrinter {
    /**
     * Prints the record. The resulting prints is based on the provided
     * templates.
     * @param record the record to provide values for variables in the 
     * templates
     * @param templates the template texts (file names), one per print
     * @throws IOException iff an I/O error occurs
     * @see org.kaiwitte.workbench.tools.Tools#replaceVariables
     */
    public void printRecord(MetaRecord record,
                            UIConfig config,
                            Sequence<String> templates)
            throws IOException {
        for (String resource : templates) {
            String content = Tools.readFile(resource);
            content = Tools.replaceVariables(record, content, config);

            DocumentRenderer renderer = new DocumentRenderer();
            JEditorPane text = new JEditorPane();
            text.setContentType("text/html");
            text.setText(content);
            renderer.print(text);
        }
    }
}
