package org.kaiwitte.workbench.ui;

import org.kaiwitte.workbench.ui.view.MainView;
import org.kaiwitte.workbench.ui.model.MainModel;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.data.*;

import java.io.IOException;

import net.tmorris.adt.sequence.Sequence;

/**
 * An interface to hold configuration data for a certain runtime context.
 */
public interface UIConfig {
    /**
     * Returns the main view of the application.
     * @return the main view of the application
     */
    public MainView getMainView();

    /**
     * Returns the main model of the application.
     * @return the main model of the application
     */
    public MainModel getMainModel();
    
    public MetaDataManagerFactory getMetaDataManagerFactory();

    /**
     * Returns the <code>MetaDataManager</code> to be used by all
     * components.
     * @return the <code>MetaDataManager</code> to access data
     */
    public MetaDataManager getMetaDataManager(MetaScheme scheme)
        throws IOException;

    /**
     * Returns the <code>MetaDataManager</code> to be used by all
     * components.
     * @return the <code>MetaDataManager</code> to access data
     */
    public Sequence<MetaSchemeTableModel> getMetaSchemeTableModels();

    /**
     * Returns a resource value. Used for string data that is visible by
     * the user somehow, for example messages in dialogs, labels and so on.
     * @return the value for the key
     * @param key the key to access the data
     */
    public String getResource(String key);

    /**
     * Returns the <code>DataManager</code> to be used by all
     * components.
     * @return the <code>DataManager</code> to access data
     */
    public DataManager getDataManager(DataScheme scheme)
        throws IOException;

    /**
     * Returns the value for a certain option. The options obtained here
     * are configurable by the user.
     * @param key the key for the option
     * @return the value
     */
    public String getOption(String key);
}
