package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.data.MetaRecord;
import org.kaiwitte.workbench.data.MetaScheme;
import org.kaiwitte.workbench.data.Record;
import org.kaiwitte.workbench.ui.model.RecordFactory;

import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * An action to apply a template to a {@link RecordReceiver}. The
 * templates can be defined by the user in the option dialog.
 * @see org.kaiwitte.workbench.ui.view.QuickSearchPanel
 * @see org.kaiwitte.workbench.ui.view.NewRecordPanel 
 */ 
public final class ApplyPrototypeAction extends AbstractAction {
    private final MetaScheme scheme;
    private final RecordReceiver receiver;
    private final String recordString;

    /**
     * Creates a new instance.
     * @param scheme the scheme to be used
     * @param name the name for this <code>Action</code>
     * @param receiver the receiver of the prototype record
     * @param recordString the string that defines the fields of the
     * template record. Fields are separated by commata.
     */ 
    public ApplyPrototypeAction(MetaScheme scheme,
                                String name,
                                RecordReceiver receiver, 
                                String recordString) {
        this.scheme = scheme;
        this.receiver = receiver;
        this.recordString = recordString;
        putValue(Action.NAME, name);
    }

    /**
     * Applies the template to the receiver.
     * @param e not used
     */ 
    public void actionPerformed(ActionEvent e) {
        Record r = RecordFactory.createRecord(scheme.getDataSchemes().get(0), 
                                              recordString);
        receiver.receiveRecord(r);
    }
}
