package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.model.TableSorter;
import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.data.MetaRecord;
import org.kaiwitte.workbench.data.MetaDataManager;
import org.kaiwitte.workbench.tools.Tools;
import org.kaiwitte.tools.SwingTools;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;

import static java.lang.System.*;

/**
 * Deletes a record.
 */ 
public final class DeleteAction extends AbstractAction {
    private final JTable table;
    private final UIConfig config;

    /**
     * Creates a new instance.
     * @param table the table
     * @param config the <code>UIConfig</code> of the runtime context
     */ 
    public DeleteAction(JTable table, UIConfig config) {
        this.table = table;
        this.config = config;
        putValue(Action.NAME, config.getResource("delete"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Delete24.gif")));
    }

    /**
     * Removes the selected records from the table.
     * @param e not used
     */ 
    public void actionPerformed(ActionEvent e) {
        if (!config.getMainView().confirm(
                config.getResource("confirm_delete_records"))) {
            return;
        }
        final MetaSchemeTableModel model = 
            ((TableSorter) table.getModel()).getTableModel();
        final java.util.List<MetaRecord> records = 
            new ArrayList<MetaRecord>();
        for (int index : table.getSelectedRows()) {
            MetaRecord record = model.getRecord(index);
            records.add(record);
        }

        Runnable runnable = new Runnable() {
            public void run() {
                for (MetaRecord record : records) {
                    try {
                        MetaDataManager data = 
                            config.getMetaDataManager(model.getScheme());
                        data.delete(record.getRecords().get(0).getID());
                        model.delete(record);
                    } catch (IOException e1) { 
                        config.getMainModel().logException(e1);
                        config.getMainView().error(
                            config.getResource("error_deleting_from_database"), 
                            e1);
                    }
                }
            }
        };
        SwingTools.launchWithProgressDialog(runnable, 
                                       config.getResource(
                                           "wait_message_delete"),
                                       config.getMainView());
        
    }
}
