package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.view.SearchPanel;
import org.kaiwitte.workbench.ui.view.NewRecordPanel;
import org.kaiwitte.workbench.ui.view.EditRecordPanel;
import org.kaiwitte.workbench.ui.view.MailDialog;
import org.kaiwitte.workbench.data.*;
import org.kaiwitte.workbench.mail.RecordMailer;
import org.kaiwitte.workbench.mail.RecordMailerImpl;
import org.kaiwitte.workbench.mail.MailCreatorImpl;
import static java.lang.System.out;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.io.IOException;
import java.sql.SQLException;

/**
 * Changes a record.
 */ 
public class EditRecordAction extends AbstractAction {
    private final UIConfig config;
    private final EditRecordPanel editRecordPanel;
    private final MetaSchemeTableModel model;
    private final int index;
    private final JDialog dialog;

    /**
     * Creates a new instance.
     * @param model the table model
     * @param config the <code>UIConfig</code> of the runtime context
     * @param index the index of the record to be changed
     * @param editPanel the panel with the new data
     */
    public EditRecordAction(UIConfig config,
                            MetaSchemeTableModel model,
                            int index,
                            EditRecordPanel editPanel,
                            JDialog dialog) {
        this.model = model;
        this.index = index;
        this.dialog = dialog;
        putValue(Action.NAME, config.getResource("edit"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Edit24.gif")));
        this.config = config;
        this.editRecordPanel = editPanel;
    }

    /**
     * Performs the editing.
     * @param e not used
     */ 
    public void actionPerformed(ActionEvent e) {
        MetaRecord mr = null;
        try {
            MetaDataManager manager = 
                config.getMetaDataManager(
                    editRecordPanel.getTableModel().getScheme());
            mr = editRecordPanel.createMetaRecord();
            manager.update(mr);
            model.update(mr, index);
            if (new Boolean(config.getOption("close_after_edit"))) {
                dialog.setVisible(false);
            }
            //newRecordPanel.getTableModel().addRecord(mr);
        } catch (IOException e1) { config.getMainModel().logException(e1);
            config.getMainView().error(
                config.getResource(
                    "error_connecting_to_or_writing_to_database"),
                e1);
            return;
        }
        try {
            checkMailCondition(mr);
        } catch (IOException e1) { config.getMainModel().logException(e1);
            config.getMainView().error(
                config.getResource("error_sending_mail"), e1);
        }
    }

    private void checkMailCondition(MetaRecord mr) throws IOException {
        int field = 0;
        StringBuilder awaitedStatus = new StringBuilder();
        for (Record record : mr.getRecords()) {
            final DataScheme scheme = record.getScheme();
            awaitedStatus.append(scheme.getMailCondition());
            for (DataColumn column : scheme.getColumns()) {
                StringBuilder status = new StringBuilder();
                status.append(mr.getColumnName(field));
                status.append("=");
                status.append(mr.getValue(field));
                if (awaitedStatus.toString().equals(status.toString())) {
                    sendMail(mr);
                    return;
                }
                if (column.getAssociatedDataScheme() == null) {
                    field++;
                }
            }
        }
    }

    private void sendMail(MetaRecord mr) throws IOException {
        RecordMailer mailer = new RecordMailerImpl();
        String[][] mails = 
            new MailCreatorImpl().createMail(
                mr, 
                config, 
                "options/mail-" 
                    + mr.getRecords().get(0).getScheme().getName() 
                    + ".txt");
        
        for (String[] mail : mails) {
            new MailDialog(config.getMainView(),
                           mail[0],
                           mail[1],
                           config.getOption("subject"),
                           mail[2],
                           config).setVisible(true);
        }
    }
}
