package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.view.NewRecordPanel;
import org.kaiwitte.workbench.ui.view.MetaRecordCreator;
import org.kaiwitte.workbench.printing.RecordPrinter;
import org.kaiwitte.workbench.printing.RecordPrinterImpl;
import org.kaiwitte.workbench.printing.PrintTemplateCreator;
import org.kaiwitte.workbench.data.MetaRecord;
import static java.lang.System.out;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.io.IOException;

import net.tmorris.adt.sequence.Sequence;

/**
 * Prints to paper based on the template file and a {@link MetaRecord}.
 */
public class PrintAction extends AbstractAction {
    private final UIConfig config;
    private final MetaRecordCreator metaRecordCreator;
    private final PrintTemplateCreator templates;
    private final boolean closeDialog;
    private final JDialog dialog;

    /**
     * Creates a new instance.
     * @param metaRecordCreator the panel with the {@link MetaRecord} to
     * be printed
     * @param config the <code>UIConfig</code> of the runtime context
     */
    public PrintAction(UIConfig config,
                       MetaRecordCreator metaRecordCreator,
                       PrintTemplateCreator templates,
                       boolean closeDialog,
                       JDialog dialog) {
        this.config = config;
        this.metaRecordCreator = metaRecordCreator;
        this.templates = templates;
        this.closeDialog = closeDialog;
        this.dialog = dialog;
        super.putValue(Action.NAME, config.getResource("print"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Print24.gif")));
    }

    /**
     * Performs the printing.
     * @param e not used
     */
    public void actionPerformed(ActionEvent e) {
        RecordPrinter printer = new RecordPrinterImpl();
        MetaRecord record = metaRecordCreator.createMetaRecord();

        try {
            Sequence<String> filenames = templates.templates();
            if (filenames == null) {
                config.getMainView().error(
                    config.getResource("no_printtemplates")
                );
            }
            else {
                printer.printRecord(record, config, filenames);
                dialog.setVisible(!closeDialog);
            }
        } catch (IOException ioe) { config.getMainModel().logException(ioe);
            config.getMainView().error(
                config.getResource("error_printing_record"),
                ioe);
        }
    }
}
