package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.model.SchemeTableModel;
import org.kaiwitte.workbench.data.Record;

import javax.swing.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

/**
 * A <code>MouseListener</code> implementation for a <code>JTable</code>
 * that hands out the {@link Record} in a {@link SchemeTableModel} to
 * a {@link RecordReceiver}.
 */ 
public class RecordTransferListener implements MouseListener {
    private final JTable table;
    private final SchemeTableModel model;
    private final RecordReceiver receiver;

    /**
     * Creates a new instance.
     * @param table the table
     * @param model the <code>SchemeTableModel</code> of the table
     * @param receiver the <code>RecordReceiver</code> that receives the
     * record from the table
     */ 
    public RecordTransferListener(JTable table, 
                                  SchemeTableModel model,
                                  RecordReceiver receiver) {
        this.table = table;
        this.model = model;
        this.receiver = receiver;
    }

    /**
     * Triggers the receiving.
     * @param e the mouse event
     */ 
    public void mouseClicked(MouseEvent e) {
        transfer(e);
    }

    private void transfer(MouseEvent e) {
        int index = table.getSelectedRow();
        if (index > -1) {
            Record record = model.getRecord(index);
            receiver.receiveRecord(record);
        }
    }

    /**
     * Does nothing.
     * @param e not used
     */ 
    public void mousePressed(MouseEvent e) {
    }

    /**
     * Does nothing.
     * @param e not used
     */ 
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Does nothing.
     * @param e not used
     */ 
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * Does nothing.
     * @param e not used
     */ 
    public void mouseExited(MouseEvent e) {
    }
}
