package org.kaiwitte.workbench.ui.controller;

import static java.lang.System.*;

import javax.swing.*;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

/**
 * Remaps the enter-key. For any <code>JComponent</code> it makes the
 * enter-key focus on the next registered <code>JComponent</code>.
 * For any <code>JButton</code> it calls its <code>Action</code> in addition. 
 */ 
public final class ReturnKeyRemapper {
    private final java.util.List<JComponent> components =
        new ArrayList<JComponent>();
    /**
     * Creates a new instance.
     */ 
    public ReturnKeyRemapper() {
    }
    
    /**
     * Remaps the enter-key for <code>component</code>. Pressing enter
     * on it results in the next component (registered with the following
     * call of this method) to be focused if there is such a component.
     * @param component the component
     * @throws NullPointerException iff <code>component == null</code>
     */ 
    public void remap(final JComponent component) {
        components.add(component);
        if (component instanceof JTextArea) {
            return;
        }
        Action enterAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                int index = components.indexOf(e.getSource());
                assert index != -1;
                if (component instanceof JButton) {
                    JButton button = (JButton) component;
                    button.getAction().actionPerformed(null);
                }
                if (index + 1 < components.size()
                        && !(component instanceof JTextArea)) {
                    JComponent nextComponent = components.get(index + 1);
                    nextComponent.grabFocus();
                }
            }
        };

        KeyStroke enter = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false);
        component.getInputMap(
            JComponent.WHEN_FOCUSED).put(enter, "ENTER");
        component.getActionMap().put("ENTER", enterAction);
    }
}
