package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.view.SearchPanel;
import org.kaiwitte.workbench.data.DataColumn;
import org.kaiwitte.workbench.data.Condition;
import org.kaiwitte.workbench.data.MetaDataManager;
import org.kaiwitte.workbench.data.MetaRecord;
import org.kaiwitte.workbench.tools.Tools;
import org.kaiwitte.tools.SwingTools;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.io.IOException;
import java.sql.SQLException;

import net.tmorris.adt.sequence.Sequence;

/**
 * Performs a search in the database.
 */
public class SearchAction extends AbstractAction {
    private final UIConfig config;
    private final SearchPanel searchPanel;

    /**
     * Creates a new instance.
     * @param searchPanel the search panel
     * @param config the <code>UIConfig</code> of the runtime context
     */
    public SearchAction(UIConfig config, SearchPanel searchPanel) {
        putValue(Action.NAME, config.getResource("start_search"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Find24.gif")));
        this.config = config;
        this.searchPanel = searchPanel;
    }

    /**
     * Performs the search.
     * @param e not used
     */
    public void actionPerformed(ActionEvent e) {
        Runnable runnable = new Runnable() {
            public void run() {
                try {
                    MetaDataManager manager =
                        config.getMetaDataManager(
                            searchPanel.getTableModel().getScheme());
                    Condition c = searchPanel.createCondition();
                    Sequence<MetaRecord> results = manager.find(c);
                    searchPanel.getTableModel().clear();
                    searchPanel.getTableModel().addRecord(
                        Tools.sequenceToArray(results, MetaRecord[].class)
                    );
                } catch (IOException e1) { config.getMainModel().logException(e1);
                    config.getMainView().error(
                        config.getResource(
                            "error_connecting_to_or_reading_from_database"),
                        e1);
                }
            }
        };
        SwingTools.launchWithProgressDialog(runnable,
                                            config.getResource(
                                                "wait_message_search"),
                                            config.getMainView());

    }
}
