package org.kaiwitte.workbench.ui.controller;

import static java.lang.System.*;

import org.kaiwitte.workbench.ui.view.MailDialog;
import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.tools.Tools;
import org.kaiwitte.tools.SwingTools;

import javax.swing.*;
import javax.mail.Session;
import javax.mail.Message;
import javax.mail.Transport;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.InternetAddress;
import java.awt.event.ActionEvent;
import java.util.Properties;
import java.util.Date;
import java.io.IOException;

/**
 * Sends an email.
 */ 
public final class SendMailAction extends AbstractAction {
    private final MailDialog mailDialog;
    private final UIConfig config;

    /**
     * Creates a new instance.
     * @param mailDialog the dialog from where to receive the mail data
     * @param config the <code>UIConfig</code> of the runtime context
     */ 
    public SendMailAction(MailDialog mailDialog,
                          UIConfig config) {
        this.mailDialog = mailDialog;
        this.config = config;
        putValue(Action.NAME, config.getResource("send"));
    }

    /**
     * Sends the mail.
     * @param e not used
     */ 
    public void actionPerformed(ActionEvent e) {
        Runnable runnable = new Runnable() {
            public void run() {
                try {
                    String from = mailDialog.getFrom();
                    String to = mailDialog.getTo();
                    String subject = mailDialog.getSubject();
                    String body = mailDialog.getMessageBody();
                    String host = config.getOption("smtphost");
    
                    boolean debug = false;
                    // create some properties and get the default Session
                    Properties props = new Properties();
                    props.put("mail.smtp.host", host);
                    props.put("mail.debug", Boolean.toString(debug));
    
                    Session session = Session.getInstance(props, null);
                    session.setDebug(debug);
                                
                    // create a message
                    Message msg = new MimeMessage(session);
                    msg.setFrom(new InternetAddress(from));
                    InternetAddress[] address = {new InternetAddress(to)};
                    msg.setRecipients(Message.RecipientType.TO, address);
                    msg.setSubject(subject);
                    msg.setSentDate(new Date());
                    
                    // If the desired charset is known, you can use
                    // setText(text, charset)
                
                    msg.setText(body);
                    Transport.send(msg);
                    config.getMainView().info(
                        config.getResource("mail_sent_successfully"));
                    mailDialog.setVisible(false);
                } catch (MessagingException me) { 
                    config.getMainModel().logException(me);
                    config.getMainView().error(
                        config.getResource("error_sending_mail"),
                        me
                    );
                } 
            }
        };
        SwingTools.launchWithProgressDialog(runnable, 
                                            config.getResource("sending_mail"),
                                            mailDialog);
    }
}
