package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.view.SearchPanel;
import org.kaiwitte.workbench.ui.view.NewRecordPanel;
import org.kaiwitte.workbench.ui.view.EditRecordPanel;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.data.MetaRecord;

import javax.swing.*;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.*;

/**
 * An <code>Action</code> to show a dialog to edit a {@link MetaRecord}.
 */ 
public final class ShowEditDialogAction extends AbstractAction {
    private final UIConfig config;
    private final MetaSchemeTableModel tableModel;
    private final JTable table;

    /**
     * Creates a new instance.
     * @param table the table
     * @param tableModel the table model
     * @param config the <code>UIConfig</code> of the runtime context
     */ 
    public ShowEditDialogAction(UIConfig config, 
                                MetaSchemeTableModel tableModel,
                                JTable table) {
        this.config = config;
        this.tableModel = tableModel;
        this.table = table;
        putValue(Action.NAME, config.getResource("edit_record"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Edit24.gif")));
    }

    /**
     * Shows the edit dialog for the selected record.
     * @param e not used
     */ 
    public void actionPerformed(ActionEvent e) {
        final JDialog searchDialog = 
            new JDialog(config.getMainView(), 
                        config.getResource("edit_record"));
        int index = table.getSelectedRow();
        if (index < 0) {
            index = 0;
        }
        MetaRecord record = tableModel.getRecord(index);
        EditRecordPanel panel = new EditRecordPanel(tableModel, index, 
                                                    config, record, 
                                                    searchDialog);

        // escape closes dialog
        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        Action escapeAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                searchDialog.dispose();
            }
        };
        searchDialog.getRootPane().getInputMap(
            JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
        searchDialog.getRootPane().getActionMap().put("ESCAPE", escapeAction);

        searchDialog.setFocusTraversalPolicy(panel.getPolicy());
        searchDialog.setLayout(new BorderLayout());
        searchDialog.add(panel);
        searchDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
//        searchDialog.setSize(400, 400);
        searchDialog.pack();
        searchDialog.setModal(true);
        searchDialog.setVisible(true);
    }
}
