package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.view.SearchPanel;
import org.kaiwitte.workbench.ui.view.NewRecordPanel;
import org.kaiwitte.workbench.ui.view.QuickSearchPanel;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.model.SchemeTableModel;
import org.kaiwitte.workbench.data.MetaDataManager;
import org.kaiwitte.workbench.data.Condition;
import org.kaiwitte.workbench.data.ConditionFactory;

import static java.awt.GridBagConstraints.*;
import static java.lang.System.*;

import javax.swing.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.*;
import java.io.IOException;

import net.tmorris.adt.sequence.Sequence;

/**
 * An <code>Action</code> to display a dialog for entering a new record.
 */
public final class ShowNewRecordDialog extends AbstractAction {
    private final UIConfig config;
    private final MetaSchemeTableModel table;

    /**
     * Creates a new instance.
     * @param table the table model
     * @param config the <code>UIConfig</code> of the runtime context
     */
    public ShowNewRecordDialog(UIConfig config, MetaSchemeTableModel table) {
        this.config = config;
        this.table = table;
        putValue(Action.NAME, config.getResource("new_record"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/New24.gif")));
    }

    /**
     * Shows the dialog.
     * @param e not used
     */
    public void actionPerformed(ActionEvent e) {
        final JDialog newRecordDialog = createNewRecordDialog();

        final NewRecordPanel newRecordPanel =
            new NewRecordPanel(table, config, newRecordDialog);
        final QuickSearchPanel quickSearchPanel =
            createQuickSearchPanel(newRecordPanel, newRecordDialog);

        final FocusTraversalPolicy policy = newRecordPanel.getPolicy();
        newRecordDialog.setFocusTraversalPolicy(policy);

        /*
        * layout components
        */
        //newRecordDialog.setLayout(new GridBagLayout());
        JPanel newRecordDialogPanel = new JPanel();
        newRecordDialogPanel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        JLabel onlineIndicator;
        try {
            MetaDataManager manager =
                config.getMetaDataManager(this.table.getScheme());
            Condition cond = ConditionFactory.rawSQLCondition("-1 = 1");
            manager.find(cond);
            onlineIndicator = new JLabel(config.getResource("database_online"),
                                         ColorCellRenderer.GREEN,
                                         JLabel.LEFT);
        } catch (IOException ioe) {
            onlineIndicator = new JLabel(config.getResource("database_offline"),
                                         ColorCellRenderer.RED,
                                         JLabel.LEFT);
        }

        c.gridy = 0;
        c.gridwidth=1;
        newRecordDialogPanel.add(onlineIndicator, c);
        
        c.gridy++;
        c.gridx = 0;
        c.weightx = 0.1;
        c.weighty = 0.0;
        c.gridwidth = 1;
        c.anchor = NORTH;
        //c.gridheight = 2;

        newRecordDialogPanel.add(createPrototypePanel(newRecordPanel), c);
        //newRecordDialogPanel.add(new JScrollPane(new JTextArea()), c);

        c.gridx = 1;
        c.gridy = 0;
        JLabel topic = new JLabel(config.getResource("new_entry_topic"));
        topic.setFont(new Font(null, Font.PLAIN, 20));
        newRecordDialogPanel.add(topic, c);
        c.gridy++;

        c.gridx = 1;
        c.weightx = 1.0;
        c.weighty = 0.15;
        c.fill = BOTH;
        c.anchor = NORTH;
        newRecordDialogPanel.add(newRecordPanel, c);
//        newRecordDialogPanel.add(new JScrollPane(new JTextArea()), c);

        c.gridy++;
        c.weighty = 0.0;
        newRecordDialogPanel.add(new JLabel(
            config.getResource("similar_records")), c);

        c.gridy++;
        c.weighty = 0.3;
        c.weightx = 5.0;
        c.fill = BOTH;
        c.anchor = CENTER;

//        JFrame testframe = new JFrame();
//        testframe.add(quickSearchPanel);
//        testframe.pack();
//        testframe.setVisible(true);
        newRecordDialogPanel.add(quickSearchPanel, c);
//        newRecordDialogPanel.add(new JTextArea("test"), c);
        newRecordDialog.add(
            new JScrollPane(newRecordDialogPanel,
                            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));

        newRecordDialog.pack();
        newRecordDialog.setVisible(true);
    }

    private JDialog createNewRecordDialog() {
        final JDialog searchDialog =
            new JDialog(config.getMainView(),
                        config.getResource("new_record"), true);

        // escape closes dialog
        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        Action escapeAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                searchDialog.dispose();
            }
        };
        searchDialog.getRootPane().getInputMap(
            JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
        searchDialog.getRootPane().getActionMap().put("ESCAPE", escapeAction);

        searchDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        return searchDialog;
    }

    private JPanel createPrototypePanel(RecordReceiver prototypeReceiver) {
        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.fill = HORIZONTAL;
        JPanel prototypePanel = new JPanel(layout);
        int prototypeNumber = 0;
        for (char a = 'a'; a <= 'd'; a++) {
            prototypeNumber++;
            if (!(config.getOption("datascheme" + a)
                    .equals(
                    table.getScheme().getDataSchemes().get(0).getName()))) {
                continue;
            }
            String recordString = createRecordString(a);

            Action prototypeAction =
                new ApplyPrototypeAction(table.getScheme(),
                                         prototypeNumber + " "
                                         + config.getOption("name"+ a),
                                         prototypeReceiver,
                                         recordString);
            JButton button = new JButton(prototypeAction);
            button.setMnemonic(Integer.toString(prototypeNumber).charAt(0));
            prototypePanel.add(button, c);
            c.gridy++;
        }
        JButton reconnect = new JButton(new ReconnectAction(config));
        prototypePanel.add(reconnect, c);

        
        return prototypePanel;
    }

    private QuickSearchPanel createQuickSearchPanel(final NewRecordPanel panel,
                                                    JDialog owner) {
        final QuickSearchPanel quickSearchPanel =
            new QuickSearchPanel(table.getScheme(), config, owner);

        Sequence<JTable> tables = quickSearchPanel.getTables();

        for (JTable jt : tables) {
            SchemeTableModel stm = (SchemeTableModel) jt.getModel();
            jt.addMouseListener(new RecordTransferListener(jt, stm, panel));
        }

        panel.addDocumentListener(
            new DocumentListener() {
                public void insertUpdate(DocumentEvent e) {
                    search();
                }

                public void removeUpdate(DocumentEvent e) {
                    search();
                }

                public void changedUpdate(DocumentEvent e) {
                    search();
                }
                private void search() {
                    quickSearchPanel.performSearch(panel.createMetaRecord());
                }
            }
        );
        return quickSearchPanel;
    }

    private String createRecordString(char a) {
        StringBuilder ret = new StringBuilder();
//        if (!(config.getOption("datascheme").equals(
//                table.getScheme().getDataSchemes().get(0).getName()))) {
//            out.println("wrong scheme: " + config.getOption("datascheme"));
//            out.println("for: " + table.getScheme().getDataSchemes().get(0));
//            
//            return "";
//        }
        for (int i = 1; i <= 15; i++) {
            ret.append(config.getOption("field" + i + a).replaceAll(",", ";"));
            ret.append(",");
        }
        //out.println(ret);
        return ret.toString();
    }
}
