package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.view.SearchPanel;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.*;
import java.net.URL;

/**
 * An <code>Action</code> to display a search dialog.
 * @see SearchPanel
 */ 
public final class ShowSearchDialog extends AbstractAction {
    private final UIConfig config;
    private final MetaSchemeTableModel table;

    /**
     * Creates a new instance.
     * @param table the table model
     * @param config the <code>UIConfig</code> of the runtime context
     */ 
    public ShowSearchDialog(UIConfig config, MetaSchemeTableModel table) {
        this.config = config;
        this.table = table;
        putValue(Action.NAME, config.getResource("search"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Find24.gif")));
    }

    /**
     * Shows the dialog.
     * @param e not used
     */ 
    public void actionPerformed(ActionEvent e) {
        final JDialog searchDialog = 
            new JDialog(config.getMainView(), config.getResource("search"));
        
        SearchPanel panel = new SearchPanel(table, config);
        searchDialog.setFocusTraversalPolicy(panel.getPolicy());
        // escape closes dialog
        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        Action escapeAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                searchDialog.dispose();
            }
        };

        searchDialog.getRootPane().getInputMap(
            JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
        searchDialog.getRootPane().getActionMap().put("ESCAPE", escapeAction);
        searchDialog.setLayout(new BorderLayout());
        searchDialog.add(panel);
        searchDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
//        searchDialog.setSize(400, 400);
        searchDialog.pack();
        searchDialog.setModal(true);
        searchDialog.setVisible(true);
    }
}
