package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.view.SearchPanel;
import org.kaiwitte.workbench.ui.view.NewRecordPanel;
import org.kaiwitte.workbench.ui.view.QuickSearchPanel;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.model.SchemeTableModel;
import org.kaiwitte.workbench.data.MetaRecord;
import static java.awt.GridBagConstraints.*;
import static java.lang.System.*;

import javax.swing.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.*;

/**
 * An <code>Action</code> to display to copy a {@link MetaRecord} from one
 * table to an other.
 */ 
public final class ShowTransferRecordDialog extends AbstractAction {
    private final UIConfig config;
    private final MetaRecord selectedRecord;
    private final MetaSchemeTableModel sourceTableModel;
    private final MetaSchemeTableModel targetTableModel;

    /**
     * Creates a new instance.
     * @param config the <code>UIConfig</code> of the runtime context
     * @param selectedRecord the record to be transfered
     * @param sourceTableModel the source location
     * @param sourceTableModel the destination
     */ 
    public ShowTransferRecordDialog(UIConfig config,
                                    MetaRecord selectedRecord, 
                                    MetaSchemeTableModel sourceTableModel,
                                    MetaSchemeTableModel targetTableModel) {
        this.config = config;
        this.selectedRecord = selectedRecord;
        this.sourceTableModel = sourceTableModel;
        this.targetTableModel = targetTableModel;
        putValue(Action.NAME, targetTableModel.getScheme().getName());
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Export24.gif")));
    }

    /**
     * Shows the dialog.
     * @param e not used
     */ 
    public void actionPerformed(ActionEvent e) {
        final JDialog searchDialog = 
            new JDialog(config.getMainView(), 
                        config.getResource("transfer_record_from")
                        + " "
                        + sourceTableModel.getScheme().getName()
                        + " " + config.getResource("to") + " "
                        + targetTableModel.getScheme().getName());
        
        // escape closes dialog
        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        Action escapeAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                searchDialog.dispose();
            }
        };
        searchDialog.getRootPane().getInputMap(
            JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
        searchDialog.getRootPane().getActionMap().put("ESCAPE", escapeAction);

        final NewRecordPanel panel = new NewRecordPanel(targetTableModel, 
                                                        config,
                                                        searchDialog);
        
        panel.receiveMetaRecord(selectedRecord, sourceTableModel.getScheme());
        
        final FocusTraversalPolicy policy = panel.getPolicy();
        searchDialog.setFocusTraversalPolicy(policy);
        
        /*
         * layout components
         */ 
        searchDialog.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        
        c.gridy = 0;
        c.gridx = 0;
        //c.gridheight = 2;

        for (char a = 'a'; a < 'c'; a++) {
            if (!(config.getOption("datascheme" + a)
                    .equals(
                    targetTableModel.getScheme().getDataSchemes().get(0).getName()))) {
                continue;
            }
            String recordString = createRecordString(a);

            Action prototypeAction = 
                new ApplyPrototypeAction(targetTableModel.getScheme(),
                                         config.getOption("name" + a),
                                         panel, 
                                         recordString);
            searchDialog.add(new JButton(prototypeAction), c);
        }

        c.gridx = 1;
        c.gridy++;
        c.fill = BOTH;
        searchDialog.add(panel, c);
        
        searchDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        searchDialog.pack();
        searchDialog.setModal(true);
        searchDialog.setVisible(true);
    }

    private String createRecordString(char a) {
        StringBuilder ret = new StringBuilder();
//        if (!(config.getOption("datascheme").equals(
//                table.getScheme().getDataSchemes().get(0).getName()))) {
//            out.println("wrong scheme: " + config.getOption("datascheme"));
//            out.println("for: " + table.getScheme().getDataSchemes().get(0));
//            
//            return "";
//        }
        for (int i = 1; i <= 15; i++) {
            ret.append(config.getOption("field" + i + a).replaceAll(",", ";"));
            ret.append(",");
        }
        //out.println(ret);
        return ret.toString();
    }
}
