package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModelFactory;
import org.kaiwitte.workbench.ui.view.SearchPanel;
import org.kaiwitte.workbench.ui.view.NewRecordPanel;
import org.kaiwitte.workbench.data.*;
import org.kaiwitte.workbench.tools.Tools;
import org.kaiwitte.tools.SwingTools;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.io.IOException;
import java.sql.SQLException;
import static java.lang.System.out;

/**
 * An <code>Action</code> to store a record in the persistence layer.
 */ 
public class StoreRecordAction extends AbstractAction {
    private final UIConfig config;
    private final NewRecordPanel newRecordPanel;
    
    private MetaRecord insertedRecord;
    private final JDialog dialog;

    /**
     * Creates a new instance.
     * @param newRecordPanel the panel where the user entered the data
     * @param config the <code>UIConfig</code> of the runtime context
     */
    public StoreRecordAction(UIConfig config,
                             NewRecordPanel newRecordPanel,
                             JDialog dialog) {
        this.dialog = dialog;
        putValue(Action.NAME, config.getResource("store"));
        putValue(Action.SMALL_ICON, new ImageIcon(
            getClass().getResource("/res/Save24.gif")));
        this.config = config;
        this.newRecordPanel = newRecordPanel;
    }

    public MetaRecord getInsertedRecord() {
        return insertedRecord;
    }

    /**
     * Performs the storing.
     * @param e not used
     */ 
    public void actionPerformed(final ActionEvent e) {
        Runnable runnable = new Runnable() {
            public void run() {
                try {
                    MetaDataManager manager = 
                        config.getMetaDataManager(
                            newRecordPanel.getTableModel().getScheme());
                    MetaRecord mr = newRecordPanel.createMetaRecord();
                    insertedRecord = manager.insert(mr);
                    //newRecordPanel.getTableModel().addRecord(mr);
                    if (new Boolean(config.getOption("close_after_save"))) {
                        dialog.setVisible(false);
                    }

                    // create backup record
                    DataScheme scheme = 
                        newRecordPanel.getTableModel()
                            .getScheme().getDataSchemes().get(0);
                    DataScheme copyScheme = scheme.getBackupScheme();
                    if (copyScheme != null) {
                        MetaScheme metaScheme = MetaSchemeFactory.getMetaScheme(copyScheme);
                        MetaDataManager backupDataManager =
                            config.getMetaDataManager(
                                metaScheme
                            );
                        MetaSchemeTableModel targetModel =
                            MetaSchemeTableModelFactory
                                .getMetaSchemeTableModel(metaScheme);
                        final NewRecordPanel panel = 
                            new NewRecordPanel(targetModel, 
                                               config, 
                                               new JDialog());
                        panel.receiveMetaRecord(mr, metaScheme);
                        StoreRecordAction storeBackupAction =
                            new StoreRecordAction(config, panel, new JDialog());
                        storeBackupAction.actionPerformed(e);
                    }
                } catch (IOException e1) { 
                    config.getMainModel().logException(e1);
                    config.getMainView().error(
                        config.getResource("database_write_error"), e1);
                }
            }
        };
        SwingTools.launchWithProgressDialog(runnable, 
                                       config.getResource(
                                           "wait_message_store"),
                                       config.getMainView());
    }
}
