package org.kaiwitte.workbench.ui.controller;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.model.TableSorter;
import org.kaiwitte.workbench.data.MetaRecord;

import javax.swing.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;

/**
 * A <code>MouseListener</code> to trigger the popup menu on a table with
 * {@link MetaRecord}s.
 */ 
public class TablePopupListener implements MouseListener {
    private final JTable table;
    private final UIConfig config;

    /**
     * Creates a new instance.
     * @param table the table
     * @param config the <code>UIConfig</code> of the runtime context
     */ 
    public TablePopupListener(JTable table, UIConfig config) {
        this.table = table;
        this.config = config;
    }

    public void mouseClicked(MouseEvent e) {
        checkPopupMenu(e);
    }

    private void checkPopupMenu(MouseEvent event) {
        final MetaSchemeTableModel model = 
            ((TableSorter) table.getModel()).getTableModel();
        if (event.getClickCount() == 2) {
            new ShowEditDialogAction(config, model, table).
                actionPerformed(null);
        }
        else if (event.isPopupTrigger()) {
            selectCorrectRows(event);
            JPopupMenu popup = new JPopupMenu();
            
            popup.add(new DeleteAction(table, config));
            popup.add(new ShowEditDialogAction(config, model, table));
            
            JMenu transfer = new JMenu(config.getResource("transfer_to"));
            transfer.setIcon(
                new ImageIcon(getClass().getResource("/res/Export24.gif")));
            int index = table.getSelectedRow();
            if (index > -1) {
                MetaRecord selectedRecord = model.getRecord(index);
                for (MetaSchemeTableModel m : 
                     config.getMetaSchemeTableModels()) {
//                    if (m == model) {
//                        continue;
//                    }
                    transfer.add(new ShowTransferRecordDialog(config,
                                                              selectedRecord,
                                                              model,
                                                              m));
                }
            }
            popup.add(transfer);
            
            popup.show(event.getComponent(), event.getX(), event.getY());
        }

    }

    private void selectCorrectRows(MouseEvent event) {
        int row = table.rowAtPoint(event.getPoint());
        if (row < 0) {
            return;
        }
        int[] selected = table.getSelectedRows();
        if (contains(selected, row)) {
            return;
        }
        table.getSelectionModel().setSelectionInterval(row, row);
    }

    private boolean contains(int[] selected, int row) {
        for (int i : selected) {
            if (i == row) {
                return true;
            }
        }
        return false;
    }

    public void mousePressed(MouseEvent e) {
        checkPopupMenu(e);
    }

    public void mouseReleased(MouseEvent e) {
        checkPopupMenu(e);
    }

    public void mouseEntered(MouseEvent e) {
        checkPopupMenu(e);
    }

    public void mouseExited(MouseEvent e) {
        checkPopupMenu(e);
    }
}
