package org.kaiwitte.workbench.ui.model;

import org.kaiwitte.workbench.data.Record;
import org.kaiwitte.workbench.data.MetaRecord;
import org.kaiwitte.workbench.data.MetaScheme;

import javax.swing.table.TableModel;
import java.sql.SQLException;
import java.io.IOException;

import net.tmorris.adt.sequence.Sequence;

/**
 * A <code>TableModel</code> to hold {@link MetaRecord}s.
 */
public interface MetaSchemeTableModel extends TableModel {

    /**
     * Adds a record as a table row.
     * @param r the new record
     */
    public void addRecord(MetaRecord... r);

    /**
     * Adds a record as a table row.
     * @param r the new record
     */
    public void addRecord(Sequence<MetaRecord> r);

    /**
     * Removes all entries.
     */
    public void clear();

    /**
     * Returns the scheme that defines this table model's structure.
     * @return the scheme 
     */
    public MetaScheme getScheme();

    /**
     * Returns the <code>MetaRecord</code> in the specified row.
     * @param row the row number, 0-based
     * @return the <code>MetaRecord</code> in the specified row
     * @throws IndexOutOfBoundsException iff <code>row < 0</code>
     * or <code>row ></code> the maximum index  
     */
    public MetaRecord getRecord(int row);

    /**
     * Deletes the <code>MetaRecord</code> in the specified row.
     * @param row the row number, 0-based
     * @throws IndexOutOfBoundsException iff <code>row < 0</code>
     * or <code>row ></code> the maximum index  
     */
    public void delete(int row);

    /**
     * Deletes the specified <code>MetaRecord</code>.
     * @param record a <code>MetaRecord to be deleted</code>
     */
    public void delete(MetaRecord record);

    /**
     * Updates the <code>MetaRecord</code> at the specified index.
     * @param record a <code>MetaRecord to be updated</code>
     * @param index the index of the record
     * @throws IndexOutOfBoundsException iff <code>index < 0</code>
     * or <code>index ></code> the maximum index
     */
    public void update(MetaRecord record, int index);
}
