package org.kaiwitte.workbench.ui.model;

import org.kaiwitte.workbench.data.*;
import static java.lang.System.*;

import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;

import net.tmorris.adt.sequence.Sequence;

/**
 * A class to create {@link MetaRecord}s based on a <code>String</code>
 * that holds the values for it.
 */
public final class RecordFactory {
    private static final Map<Class,StringParser> PARSERS =
        new HashMap<Class,StringParser>();

    private RecordFactory() {
    }

    /**
     * Creates a <code>Record</code> based on comma separated values.
     * @param scheme the scheme of the <code>Record</code>
     * @param s the <code>String</code>
     * @return a new <code>Record</code>
     */
    public static Record createRecord(DataScheme scheme, String s) {
        String[] strfields = s.split(",");

        final Sequence<DataColumn> columns = scheme.getColumns();
//        if (strfields.length != columns.length) {
//            throw new IllegalArgumentException(
//                "length of string: " + strfields.length + " - "
//                + "number of data columns: " + scheme.getColumns().getSize()
//                + "; must be equal"
//            );
//        }

        List values = new ArrayList();

        int j = 0;
        for (int i = 0; i < columns.getSize(); i++) {
            if (columns.get(i).getAssociatedDataScheme() != null) {
                // don't allow specification of external fields
                StringParser parser = PARSERS.get(columns.get(i).getType());
                String strValue = "";
                Object value = parser.parseString(strValue);
                values.add(value);
            }
            else {
                StringParser parser = PARSERS.get(columns.get(i).getType());

                String strValue = j < strfields.length ? strfields[j] : "";

                Object value = parser.parseString(strValue);
                values.add(value);
                j++;
            }
        }
        Record record =
            new MutableRecordImpl(scheme, values.toArray());

        return record;
    }

    private static interface StringParser {
        public Object parseString(String str);
    }

    private static final long MILLISECONDS_PER_HOUR =
        1000 * 60 * 60;

    static {
        PARSERS.put(String.class, new StringParser() {
            public Object parseString(String str) {
                return str;
            }
        });
        PARSERS.put(java.util.Date.class, new StringParser() {
            public Object parseString(String str) {
                long extraTime;

                try {
                    extraTime = Integer.parseInt(str)
                                * MILLISECONDS_PER_HOUR;
                } catch (NumberFormatException nfe) {
                    extraTime = 0;
                }
                return new java.util.Date(
                    System.currentTimeMillis() + extraTime
                );
            }
        });
        PARSERS.put(String[].class, new StringParser() {
            public Object parseString(String str) {
                return str;
            }
        });
        PARSERS.put(Boolean.class, new StringParser() {
            public Object parseString(String str) {
                return new Boolean(str);
            }
        });
        PARSERS.put(Integer.class, new StringParser() {
            public Object parseString(String str) {
                try {
                    return new Integer(str);
                } catch (NumberFormatException nfe) {
                    return new Integer(0);
                }
            }
        });
    }
}
