package org.kaiwitte.workbench.ui.model;

import org.kaiwitte.workbench.data.Record;
import org.kaiwitte.workbench.data.DataManager;
import org.kaiwitte.workbench.data.DataScheme;
import org.kaiwitte.workbench.data.DataManagerFactory;

import javax.swing.table.AbstractTableModel;
import java.util.List;
import java.util.ArrayList;

/**
 * A <code>TableModel</code> to hold 
 * {@link org.kaiwitte.workbench.data.Record}s.
 */ 
public class SchemeTableModelImpl 
        extends AbstractTableModel implements SchemeTableModel {
    private List<Record> rows = new ArrayList<Record>();
    private final DataScheme scheme;

    /**
     * Creates a new instance.
     * @param scheme the <code>DataScheme</code> that defines the
     * structure of this <code>TableModel</code>
     * @throws NullPointerException iff <code>scheme == null</code>
     */ 
    public SchemeTableModelImpl(DataScheme scheme) {
        if (scheme == null) {
            throw new NullPointerException();
        }
        this.scheme = scheme;
    }
    
    /**
     * Adds a record as a table row.
     * @param r the new record
     */ 
    public void addRecord(Record... r) {
        for (Record record : r) {
            rows.add(record);
        }
        fireTableDataChanged();
    }

    @Override 
    public String getColumnName(int column) {
        return column == 0 ?
            "ID" 
            : 
            scheme.getColumns().get(column - 1).getName();
    }

    @Override 
    public Class<?> getColumnClass(int column) {
        return column == 0 ?
            Integer.class 
            : 
            scheme.getColumns().get(column - 1).getType();
    }

    /**
     * Removes all entries.
     */ 
    public void clear() {
        rows.clear();
        fireTableDataChanged();
    }

    /**
     * Returns the <code>Record</code> in the specified row.
     * @param index the row number, 0-based
     * @return the <code>Record</code> in the specified row
     * @throws IndexOutOfBoundsException iff <code>index < 0</code>
     * or <code>index ></code> the maximum index  
     */ 
    public Record getRecord(int index) {
        return rows.get(index);
    }

    public int getRowCount() {
        return rows.size();
    }

    public int getColumnCount() {
        return scheme.getLength() + 1;
    }
    
    public Object getValueAt(int rowIndex, int columnIndex) {
        return columnIndex == 0 ?
            rows.get(rowIndex).getID() 
            : 
            rows.get(rowIndex).getValue(columnIndex - 1);
    }
}
