package org.kaiwitte.workbench.ui.model;

import static java.lang.System.*;

import java.io.OutputStream;
import java.io.IOException;

/**
 * A class to write to a buffer. Similar to <code>StringWriter</code>.
 * Thread-safe.
 */ 
public final class StringOutputStream extends OutputStream {
    private final StringBuffer buffer = new StringBuffer();
    
    /**
     * Creates a new instance.
     */ 
    public StringOutputStream() {
    }

    /**
     * Writes the specified byte to this output stream. The general
     * contract for <code>write</code> is that one byte is written
     * to the output stream. The byte to be written is the eight
     * low-order bits of the argument <code>b</code>. The 24
     * high-order bits of <code>b</code> are ignored.
     * @param b the <code>byte</code>.
     */
    public void write(int b) {
        buffer.append((char) b);
    }

    /**
     * Returns the collected data.
     * @return the collected data
     */
    @Override 
    public String toString() {
        return buffer.toString();
    }
}
