package org.kaiwitte.workbench.ui.view;

import java.awt.*;
import java.util.ArrayList;

/**
 * A <code>FocusTraversalPolicy</code> which allows to
 * {@link #add} components to the cycle order. 
 */ 
final class EditableFocusTraversalPolicy
        extends FocusTraversalPolicy {
    private final java.util.List<Component> components =
        new ArrayList<Component>();

    /**
     * Adds a component to be the next one in the cycle order. If none
     * has been added with this method yet this is the first component
     * in the order
     * @param c the next component in the cycle order
     */ 
    public void add(Component c) {
        if (components.size() == 0) {
            c.requestFocusInWindow();
        }
        components.add(c);
    }
            
    public Component getComponentAfter(Container aContainer,
                                       Component aComponent) {
        int position = components.indexOf(aComponent) + 1;
        if (position < components.size()) {
            return components.get(position);
        }
        else {
            return components.get(0);
        }
    }

    public Component getComponentBefore(Container aContainer,
                                        Component aComponent) {
        int position = components.indexOf(aComponent) - 1;
        if (position < components.size() && position > 0) {
            return components.get(position);
        }
        else {
            return components.get(components.size() - 1);
        }
    }

    public Component getFirstComponent(Container aContainer) {
        return components.get(0);
    }

    public Component getLastComponent(Container aContainer) {
        return components.get(components.size() - 1);
    }

    public Component getDefaultComponent(Container aContainer) {
        return components.get(0);
    }
}
