package org.kaiwitte.workbench.ui.view;

import static java.lang.System.*;

import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.controller.SendMailAction;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

/**
 * A dialog to write or edit an email.
 */ 
public final class MailDialog extends JDialog {
    private final JTextField fromField;
    private final JTextField toField;
    private final JTextField subjectField;
    private final JTextArea messageBody;

    private final UIConfig config;

    private final Action cancelAction = 
        new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        };

    /**
     * Creates a new instance.
     * @param owner the owner of this dialog
     * @param from the sender
     * @param to the receiver
     * @param subject the subject
     * @param body the mail body
     * @param config the <code>UIConfig</code>
     * @throws HeadlessException if GraphicsEnvironment.isHeadless() 
     * returns true.
     * @throws NullPointerException iff any argument is null
     */
    public MailDialog(Frame owner,
                      String from,
                      String to,
                      String subject,
                      String body,
                      UIConfig config) {
        super(owner, "send mail", true);
        if (owner == null || from == null || to == null || subject == null
                || body == null || config == null) {
            throw new NullPointerException();
        }
        this.config = config;

        // escape closes dialog
        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        getRootPane().getInputMap(
            JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
        getRootPane().getActionMap().put("ESCAPE", cancelAction);

        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 0.5;
        c.weighty = 0.0;

        c.gridx = 0;
        c.gridy = 0;
        JLabel froml = new JLabel("from");
        add(froml, c);
        
        c.gridx = 1;
        fromField = new JTextField(from);
        add(fromField, c);
        
        c.gridx = 0;
        c.gridy++;
        JLabel tol = new JLabel("to");
        add(tol, c);
        
        c.gridx = 1;
        toField = new JTextField(to);
        add(toField, c);
        
        c.gridx = 0;
        c.gridy++;
        JLabel subjectl = new JLabel("subject");
        add(subjectl, c);
        
        c.gridx = 1;
        subjectField = new JTextField(subject);
        add(subjectField, c);
        
        c.gridx = 0;
        c.gridwidth = 2;
        c.gridy++;
        messageBody = new JTextArea(body);
        c.weighty=1.0;
        c.fill = GridBagConstraints.BOTH;
        //messageBody.setPreferredSize(new Dimension(30, 30));
        JScrollPane bodyPanel = new JScrollPane(messageBody);
        //bodyPanel.setPreferredSize(new Dimension(400, 200));
        add(bodyPanel, c);
        
        c.gridwidth = 1;
        c.fill = GridBagConstraints.NONE;
        c.weighty = 0.0;
        c.gridy++;
        JButton ok = new JButton(new SendMailAction(this, config));
        add(ok, c);

        c.gridx = 1;
        JButton cancel = new JButton(cancelAction);
        add(cancel, c);
        
        //pack();
        setSize(400, 300);
    }
    
    /**
     * Returns the sender.
     * @return the sender
     */ 
    public String getFrom() {
        return fromField.getText();
    }
    
    /**
     * Returns the receiver.
     * @return the receiver
     */ 
    public String getTo() {
        return toField.getText();
    }
    
    /**
     * Returns the subject.
     * @return the subject
     */ 
    public String getSubject() {
        return subjectField.getText();
    }
    
    /**
     * Returns the mail body.
     * @return the body
     */ 
    public String getMessageBody() {
        return messageBody.getText();
    }
    
}
