package org.kaiwitte.workbench.ui.view;

import org.kaiwitte.workbench.data.*;
import org.kaiwitte.workbench.ui.model.*;
import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.ui.controller.SearchHistoryAction;

import static java.awt.GridBagConstraints.*;

import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.util.*;
import java.sql.SQLException;
import java.io.IOException;

import net.tmorris.adt.sequence.Sequence;
import net.tmorris.adt.sequence.ArraySequenceImpl;

/**
 * A panel that displays records that match a certain criterion.
 * @see NewRecordPanel
 * @see NewRecordPanel#receiveMetaRecord
 */
public final class QuickSearchPanel extends JPanel {
    private final MetaScheme metaScheme;
    private final UIConfig config;
    private final Map<DataScheme,SchemeTableModel> models =
        new HashMap<DataScheme,SchemeTableModel>();
    private final java.util.List<JTable> tables = new ArrayList<JTable>();
    private boolean quickSearchError;

    /**
     * Creates a new instance.
     * @param metaScheme the <code>MetaScheme</code> that defines
     * the number and type of the fields.
     * @param config the <code>UIConfig</code>
     */
    public QuickSearchPanel(MetaScheme metaScheme, UIConfig config,
                            JDialog owner) {
        this.metaScheme = metaScheme;
        this.config = config;

        Sequence<DataScheme> schemes = metaScheme.getDataSchemes();
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
//        setLayout(new GridBagLayout());
//        GridBagConstraints c = new GridBagConstraints();

//        c.gridx = 0;
//        c.gridy = 0;
        //c.fill = BOTH;

        for (int i = 1; i < schemes.getSize(); i++) {
            final DataScheme scheme = schemes.get(i);
            final JTable table = new JTable();
            final SchemeTableModel model = new SchemeTableModelImpl(scheme);
            models.put(scheme,model);
//            TableSorter sorter = new TableSorter(scheme);
//            JTable table = new JTable(sorter);
//            sorter.setTableHeader(table.getTableHeader());
            table.setModel(model);
            tables.add(table);

            /*
            * history search
            */
            ConditionCreator conditionCreator = new ConditionCreator() {
                public Condition createCondition() {
//                    int row = table.getSelectedRow();
                    int row = 0;
                    if (model.getRowCount() == 0) {
                        return ConditionFactory.rawSQLCondition(
                            "true"
                        );
                    }
                    Record record = model.getRecord(row);
                    long id = record.getID();
                    Condition c = ConditionFactory.rawSQLCondition(
                        scheme.getName() + ".id = " + id
                    );
                    return c;
                }
            };

            final JDialog resultFrame =
                new JDialog(owner, config.getResource("history_title"));
            // escape closes dialog
            KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
            Action escapeAction = new AbstractAction() {
                public void actionPerformed(ActionEvent e) {
                    resultFrame.dispose();
                }
            };
            resultFrame.getRootPane().getInputMap(
                JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
            resultFrame.getRootPane().getActionMap().put("ESCAPE", escapeAction);

            resultFrame.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

            final JTable resultTable = new JTable();
            final JScrollPane scrollPane = new JScrollPane(resultTable);
            resultFrame.getContentPane().setLayout(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            c.gridx = 0;
            c.gridy = 0;
            c.weightx = 0.0;
            c.weighty = 1.0;
            c.anchor = NORTH;
            c.fill = NONE;
            resultFrame.getContentPane().add(
                new JLabel(
                    new ImageIcon(
                        getClass().getResource("/res/singleheadsilhouette.jpg")
                    )
                ), c);
            c.fill = BOTH;
            c.weightx = 5.0;
            c.gridx++;
            c.anchor = CENTER;
            resultFrame.getContentPane().add(scrollPane, c);
            resultFrame.pack();

            MetaSchemeTableModel model2 =
                new MetaSchemeTableModelImpl(metaScheme) {
                    @Override
                    public void addRecord(MetaRecord... r) {
                        super.addRecord(r);
                        resultFrame.pack();
                        resultFrame.setVisible(true);
                    }
                    @Override
                    public void addRecord(Sequence<MetaRecord> r) {
                        super.addRecord(r);
                        resultFrame.pack();
                        resultFrame.setVisible(true);
                    }
                };
            TableSorter sorter = new TableSorter(model2);
            sorter.setTableHeader(resultTable.getTableHeader());
            resultTable.setModel(sorter);

            SearchHistoryAction sha = new SearchHistoryAction(config,
                                                              conditionCreator,
                                                              model2
            );

            JButton history = new JButton(sha);


//            c.fill = NONE;
            add(history);
//            c.gridy++;
//            c.fill = BOTH;
//            c.gridwidth = 10;
            add(new JScrollPane(table));
//            add(new JScrollPane(new JTextArea()), c);
            //c.gridy++;
            //c.gridy++;
        }
    }

    /**
     * Searches for similar records.
     * @param metaRecord the record to be compared to entries in the database
     */
    public void performSearch(MetaRecord metaRecord) {
        if (quickSearchError) {
            return;
        }
        try {
//            DataManagerFactory factory =
//                new DataManagerFactoryImpl(config.getOption("driver"),
//                                           "jdbc:mysql://"
//                                           + config.getOption("host")
//                                           + "/"
//                                           + config.getOption("databasename")
//                                           + "?user="
//                                           + config.getOption("username")
//                                           + "&password="
//                                           + config.getOption("password"));
            Sequence<DataScheme> schemes = metaScheme.getDataSchemes();
            for (int i = 1; i < schemes.getSize(); i++) {
                DataScheme scheme = schemes.get(i);
                Record criterionRecord = metaRecord.getRecords().get(i);
                DataManager manager = config.getDataManager(scheme);
                Condition c =
                    ConditionFactory.textFieldsStartWith(criterionRecord);
                if (c == null) {
                    return;
                }
                Sequence<Record> results = manager.find(c);
                models.get(scheme).clear();
                for (Record result : results) {
                    models.get(scheme).addRecord(result);
                }
            }
        } catch (IOException e) { config.getMainModel().logException(e);
            config.getMainView().error(config.getResource("quicksearch_error"),
                                       e);
            quickSearchError = true;
        }
    }

    /**
     * Returns the tables that display the results.
     * @return the tables
     */
    public Sequence<JTable> getTables() {
        return ArraySequenceImpl.getArraySequence()
                .getSequence(tables.toArray(new JTable[0]));
    }

    /**
     * A <code>TableModel</code> to hold {@link org.kaiwitte.workbench.data.MetaRecord}s.
     */
    private static class MetaSchemeTableModelImpl
            extends AbstractTableModel implements MetaSchemeTableModel {
        private java.util.List<MetaRecord> rows = new ArrayList<MetaRecord>();
        private final MetaScheme metaScheme;

        /**
         * Creates a new instance.
         * @param scheme the <code>MetaScheme</code> that defines the
         * structure of this <code>TableModel</code>
         * @throws NullPointerException iff <code>scheme == null</code>
         */
        MetaSchemeTableModelImpl(MetaScheme scheme) {
            if (scheme == null) {
                throw new NullPointerException();
            }
            this.metaScheme = scheme;
        }

        /**
         * Adds a record as a table row.
         * @param r the new record
         */
        public void addRecord(MetaRecord... r) {
            for (MetaRecord record : r) {
                rows.add(record);
            }
            fireTableDataChanged();
        }

        /**
         * Adds a record as a table row.
         * @param r the new record
         */
        public void addRecord(Sequence<MetaRecord> r) {
            for (MetaRecord record : r) {
                rows.add(record);
            }
            fireTableDataChanged();
        }

        @Override
        public String getColumnName(int column) {
            int virtualColumn = 0;
            for (int i = 0; i < metaScheme.getColumns().getSize(); i++) {
                while (metaScheme.getColumns().get(i).isHidden()) {
                    if (i >= metaScheme.getColumns().getSize()) {
                        throw new IndexOutOfBoundsException();
                    }
                    i++;
                }
                if (virtualColumn == column) {
                    return metaScheme.getColumns().get(i).getName();
                }
                virtualColumn++;
            }
            throw new IndexOutOfBoundsException();
        }

        @Override
        public Class<?> getColumnClass(int column) {
            int virtualColumn = 0;
            for (int i = 0; i < metaScheme.getColumns().getSize(); i++) {
                while (metaScheme.getColumns().get(i).isHidden()) {
                    if (i >= metaScheme.getColumns().getSize()) {
                        throw new IndexOutOfBoundsException();
                    }
                    i++;
                }
                if (virtualColumn == column) {
                    Class type = metaScheme.getColumns().get(i).getType();
                    return type.equals(String[].class) ? String.class : type;
                }
                virtualColumn++;
            }
            throw new IndexOutOfBoundsException();
        }

        /**
         * Removes all entries.
         */
        public void clear() {
            rows.clear();
            fireTableDataChanged();
        }

        /**
         * Returns the scheme that defines this table model's structure.
         * @return the scheme 
         */
        public MetaScheme getScheme() {
            return metaScheme;
        }

        /**
         * Returns the <code>MetaRecord</code> in the specified row.
         * @param row the row number, 0-based
         * @return the <code>MetaRecord</code> in the specified row
         * @throws IndexOutOfBoundsException iff <code>row < 0</code>
         * or <code>row ></code> the maximum index  
         */
        public MetaRecord getRecord(int row) {
            return rows.get(row);
        }

        /**
         * Deletes the <code>MetaRecord</code> in the specified row.
         * @param row
         * @param row the row number, 0-based
         * @throws IndexOutOfBoundsException iff <code>row < 0</code>
         * or <code>row ></code> the maximum index  
         */
        public void delete(int row) {
            rows.remove(row);
            fireTableDataChanged();
        }

        /**
         * Deletes the specified <code>MetaRecord</code>.
         * @param record a <code>MetaRecord to be deleted</code>
         */
        public void delete(MetaRecord record) {
            rows.remove(record);
            fireTableDataChanged();
        }

        /**
         * Updates the <code>MetaRecord</code> at the specified index.
         * @param record a <code>MetaRecord to be updated</code>
         * @param index the index of the record
         * @throws IndexOutOfBoundsException iff <code>index < 0</code>
         * or <code>index ></code> the maximum index
         */
        public void update(MetaRecord record, int index) {
            rows.set(index, record);
            fireTableDataChanged();
        }

        public int getRowCount() {
            return rows.size();
        }

        public int getColumnCount() {
            int currentColumn = 0;
            for (int i = 0; i < metaScheme.getColumns().getSize(); i++) {
                DataColumn dataColumn = metaScheme.getColumns().get(i);
                if (!dataColumn.isHidden()) {
                    currentColumn++;
                }
            }
            return currentColumn;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            MetaRecord record = rows.get(rowIndex);
            int virtualColumn = 0;
            for (int i = 0; i < metaScheme.getColumns().getSize(); i++) {
//            System.out.println(
//                metaScheme.getColumns().get(i).getName() + " " 
//                + metaScheme.getColumns().get(i).isHidden());
                while (metaScheme.getColumns().get(i).isHidden()) {
                    if (i >= metaScheme.getColumns().getSize()) {
                        throw new IndexOutOfBoundsException();
                    }

                    i++;
                }
                if (virtualColumn == columnIndex) {
                    return record.getValue(i);
                }
                virtualColumn++;
            }

            throw new IndexOutOfBoundsException();
        }
    }
}
