package org.kaiwitte.workbench.ui.view;

import org.kaiwitte.workbench.data.MetaScheme;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.model.TableSorter;
import org.kaiwitte.workbench.ui.controller.*;
import org.kaiwitte.workbench.ui.UIConfig;
import org.kaiwitte.workbench.tools.Tools;

import javax.swing.*;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.awt.*;
import java.util.HashMap;
import java.util.Map;

/**
 * A panel to display a table with 
 * {@link org.kaiwitte.workbench.data.MetaRecord}s.
 */ 
public class SchemeTablePanel extends JPanel {
    /**
     * Creates a new instance.
     * @param config the <code>UIConfig</code>
     * @param scheme the table model for the table
     */ 
    public SchemeTablePanel(UIConfig config, 
                            final MetaSchemeTableModel scheme) {
        super(new BorderLayout());
        
        JToolBar toolBar = new JToolBar();
        
        TableSorter sorter = new TableSorter(scheme);
        JTable table = new JTable(sorter);
        sorter.setTableHeader(table.getTableHeader());
        
        table.setDefaultRenderer(String.class, 
                                 new ColorCellRenderer(
                                     table.getDefaultRenderer(String.class),
                                     Tools.iconCreator(config)));
        final JButton searchButton = 
            new JButton(new ShowSearchDialog(config, scheme));
        toolBar.add(searchButton);
        searchButton.setMnemonic(config.getResource("search_m").charAt(0));
        final JButton newButton = 
            new JButton(new ShowNewRecordDialog(config, scheme));
        newButton.setMnemonic(config.getResource("new_m").charAt(0));
        toolBar.add(newButton);

        // edit and delete only enabled when content is in table
        final DeleteAction delete = new DeleteAction(table, config);
        scheme.addTableModelListener(
            new TableModelListener() {
                public void tableChanged(TableModelEvent e) {
                    delete.setEnabled(scheme.getRowCount() != 0);
                }
            });
        delete.setEnabled(scheme.getRowCount() != 0);
        final JButton deleteButton = new JButton(delete);
        deleteButton.setMnemonic(config.getResource("delete_m").charAt(0));
        toolBar.add(deleteButton);

        final ShowEditDialogAction edit = 
            new ShowEditDialogAction(config, scheme, table);
        scheme.addTableModelListener(
            new TableModelListener() {
                public void tableChanged(TableModelEvent e) {
                    edit.setEnabled(scheme.getRowCount() != 0);
                }
            });
        edit.setEnabled(scheme.getRowCount() != 0);
        final JButton editButton = new JButton(edit);
        editButton.setMnemonic(config.getResource("edit_m").charAt(0));
        toolBar.add(editButton);

        table.addMouseListener(new TablePopupListener(table, config));

        add(toolBar, BorderLayout.NORTH);
        add(new JScrollPane(table));
    }
}
