package test;

import org.jtiger.framework.Fixture;
import org.jtiger.framework.Category;
import org.jtiger.framework.Test;
import org.jtiger.framework.ExpectException;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertNull;

import org.kaiwitte.workbench.data.*;

import java.net.URI;
import java.net.URISyntaxException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import static java.lang.System.*;

import net.tmorris.adt.sequence.Sequence;

@Fixture("condition")
@Category("condition")
public class ConditionTest {
    private static class DataColumnImpl implements DataColumn {
        Class type;
        boolean isRequiredForUpdate;
        String name;
        DataScheme scheme;
        String nameInScheme;

        public int getTextAreaHeight() {
            throw new UnsupportedOperationException();
        }

        public Class getType() {
            return type;
        }

        public boolean isRequiredForUpdate() {
            return isRequiredForUpdate;
        }

        public String getName() {
            return name;
        }

        public DataScheme getAssociatedDataScheme() {
            return scheme;
        }

        public String getNameInAssociatedDataScheme() {
            return nameInScheme;
        }

        public Sequence<String> getChoices() {
            throw new UnsupportedOperationException();
        }

        public boolean isHidden() {
            throw new UnsupportedOperationException();
        }

        public boolean isSmall() {
            throw new UnsupportedOperationException();
        }

        public boolean isIndirectlyHidden() {
            throw new UnsupportedOperationException();
        }

        public String toString() {
            return name + " " + type;
        }
    };

    @Test("=")
    public void testEqual() {
        DataColumnImpl dc = new DataColumnImpl();
        dc.name = "location";
        dc.type = String.class;
        Condition c = ConditionFactory.columnEqual(dc, "Venezuela");
        assertEqual(
            "location='Venezuela'",
            c.toSQLString()
        );
    }

    @Test("AND")
    public void testAND() {
        DataColumnImpl dc1 = new DataColumnImpl();
        dc1.name = "location";
        dc1.type = String.class;
        Condition c1 = ConditionFactory.columnEqual(dc1, "Venezuela");

        DataColumnImpl dc2 = new DataColumnImpl();
        dc2.name = "best_friend";
        dc2.type = String.class;
        Condition c2 = ConditionFactory.columnEqual(dc2, "Tom");

        Condition and = ConditionFactory.and(c1, c2);
        assertEqual(
            "location='Venezuela' AND best_friend='Tom'",
            and.toSQLString()
        );
    }

    public static void main(String[] args) {
        new ConditionTest().testAND();
    }
}
