package test;

import static org.jtiger.framework.FixturesRunnerConfigFactory.newFixturesRunnerConfig;
import static org.jtiger.framework.ReadOnlyArrayFactory.newReadOnlyArray;
import static org.jtiger.framework.FixturesRunnerFactory.newFixturesRunner;
import org.jtiger.framework.*;
import org.jtiger.framework.Sequence;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertNull;

import org.kaiwitte.workbench.data.*;

import java.net.URI;
import java.net.URISyntaxException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

import static java.lang.System.*;

import net.tmorris.adt.sequence.*;

@Fixture("data manager")
@Category("data manager")
public class DataManagerTest {
    @Test("no search results")
    public void emptySearchResult() throws URISyntaxException, IOException,
            IllegalAccessException, SQLException, InstantiationException,
            ClassNotFoundException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemes.xml");
        net.tmorris.adt.sequence.Sequence<DataScheme> schemes = factory.createDataScheme(in);

        deleteTableContent(schemes.get(0).getName());

        DataManager data = new DataManagerFactoryImpl("com.mysql.jdbc.Driver",
                                                      "jdbc:mysql://localhost/test?user=&password=").getDataManager(
                schemes.get(0));

        Condition c = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(1), "Venezuela");

        net.tmorris.adt.sequence.Sequence<Record> results = data.find(c);

        assertEqual(0, results.getSize());

    }

    @Test("search a record")
    public void searchARecord() throws URISyntaxException, IOException,
            IllegalAccessException, SQLException, InstantiationException,
            ClassNotFoundException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemes02.xml");
        net.tmorris.adt.sequence.Sequence<DataScheme> schemes = factory.createDataScheme(in);

        deleteTableContent(schemes.get(0).getName());

        DataManager data = new DataManagerFactoryImpl("com.mysql.jdbc.Driver",
                                                      "jdbc:mysql://localhost/test?user=&password=").getDataManager(
                schemes.get(0));

        Record newRecord = new MutableRecordImpl(schemes.get(0), "Kiel", 7);

        data.insert(newRecord);

        Condition c = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(0), "Kiel");

        net.tmorris.adt.sequence.Sequence<Record> results = data.find(c);

        assertEqual(1, results.getSize());

        //Record expectedResult = new MutableRecordImpl(1, "Kiel", 7);

        assertEqual(results.get(0).getValue(0), "Kiel");
        assertEqual(results.get(0).getValue(1), 7);

    }

    @Test("delete record")
    public void deleteRecord() throws URISyntaxException, IOException,
            IllegalAccessException, SQLException, InstantiationException,
            ClassNotFoundException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemes02.xml");
        net.tmorris.adt.sequence.Sequence<DataScheme> schemes = factory.createDataScheme(in);

        deleteTableContent(schemes.get(0).getName());

        DataManager data = new DataManagerFactoryImpl("com.mysql.jdbc.Driver",
                                                      "jdbc:mysql://localhost/test?user=&password=").getDataManager(
                schemes.get(0));

        Record newRecord = new MutableRecordImpl(schemes.get(0), "Kiel", 7);

        data.insert(newRecord);

        Condition c = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(0), "Kiel");

        net.tmorris.adt.sequence.Sequence<Record> results = data.find(c);

        assertEqual(1, results.getSize());

        //Record expectedResult = new MutableRecordImpl(1, "Kiel", 7);

        assertEqual(results.get(0).getValue(0), "Kiel");
        assertEqual(results.get(0).getValue(1), 7);

        /*
        * Now delete it again
        */
        data.delete(results.get(0).getID());

        results = data.find(c);

        assertEqual(0, results.getSize());
    }

    @Test("no match")
    public void noMatch() throws URISyntaxException, IOException,
            IllegalAccessException, SQLException, InstantiationException,
            ClassNotFoundException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemes02.xml");
        net.tmorris.adt.sequence.Sequence<DataScheme> schemes = factory.createDataScheme(in);

        deleteTableContent(schemes.get(0).getName());

        DataManager data = new DataManagerFactoryImpl("com.mysql.jdbc.Driver",
                                                      "jdbc:mysql://localhost/test?user=&password=").getDataManager(
                schemes.get(0));

        Record newRecord = new MutableRecordImpl(schemes.get(0), "Kiel", 7);

        data.insert(newRecord);

        Condition c = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(0), "Kiell");

        net.tmorris.adt.sequence.Sequence<Record> results = data.find(c);

        assertEqual(0, results.getSize());
    }

    @Test("alter database entry")
    public void alterDatabaseEntry() throws URISyntaxException, IOException,
            IllegalAccessException, SQLException, InstantiationException,
            ClassNotFoundException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemes02.xml");
        net.tmorris.adt.sequence.Sequence<DataScheme> schemes = factory.createDataScheme(in);

        deleteTableContent(schemes.get(0).getName());

        DataManager data = new DataManagerFactoryImpl("com.mysql.jdbc.Driver",
                                                      "jdbc:mysql://localhost/test?user=&password=").getDataManager(
                schemes.get(0));

        Record newRecord = new MutableRecordImpl(schemes.get(0), "Kieler", 771108);

        data.insert(newRecord);

        Condition condition = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(1), 771108);

        net.tmorris.adt.sequence.Sequence<Record> results = data.find(condition);

        assertEqual(1, results.getSize());

        MutableRecord mutable = new MutableRecordImpl(results.get(0).getID(),
                                                      schemes.get(0),
                                                      results.get(0).getValue(0),
                                                      results.get(0).getValue(1));
        assertEqual(mutable, results.get(0));

        mutable.setValue(1, 78);

        data.update(mutable);

        condition = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(1), 771108);

        results = data.find(condition);

        assertEqual(0, results.getSize());

        condition = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(1), 78);

        results = data.find(condition);

        assertEqual(1, results.getSize());

        assertEqual(78, results.get(0).getValue(1));
    }

    public static void deleteTableContent(String table)
            throws ClassNotFoundException,
            IllegalAccessException, InstantiationException, SQLException {
        Class.forName("com.mysql.jdbc.Driver").newInstance();
        Connection con = DriverManager.getConnection(
            "jdbc:mysql://localhost/test?user=&password=");
        Statement stmt = con.createStatement();

        stmt.executeUpdate("DELETE from " + table);
    }
}
