package test;

import org.jtiger.framework.Fixture;
import org.jtiger.framework.Category;

import static java.lang.System.*;
import static org.jtiger.framework.FixturesRunnerConfigFactory.newFixturesRunnerConfig;
import static org.jtiger.framework.ReadOnlyArrayFactory.newReadOnlyArray;
import static org.jtiger.framework.FixturesRunnerFactory.newFixturesRunner;
import org.jtiger.framework.*;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertNull;

import org.kaiwitte.workbench.data.*;

import java.io.InputStream;
import java.io.IOException;

import net.tmorris.adt.sequence.*;
import net.tmorris.adt.sequence.Sequence;

@Fixture("meta scheme")
@Category("meta scheme")
public class MetaSchemeTest {
    @Test("meta scheme 1")
    public void testMetaScheme() throws IOException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemesconnected.xml");
        Sequence<DataScheme> schemes = factory.createDataScheme(in);
        assertEqual(2, schemes.getSize());
        DataScheme scheme1 = schemes.get(0);
        DataScheme scheme2 = schemes.get(1);

        assertEqual(2, scheme1.getLength());
        assertEqual(2, scheme1.getColumns().getSize());
        assertEqual("sells", scheme1.getName());

        assertEqual(3, scheme2.getLength());
        assertEqual(3, scheme2.getColumns().getSize());
        assertEqual("sells", scheme1.getName());

        DataColumn column11 = scheme1.getColumns().get(0);
        DataColumn column12 = scheme1.getColumns().get(1);

        assertEqual(Integer.class, column11.getType());
        assertFalse(column11.isRequiredForUpdate());
        assertEqual("Customer", column11.getName());
        assertEqual(scheme2, column11.getAssociatedDataScheme());
        assertEqual("Number", column11.getNameInAssociatedDataScheme());

        /*
        * test meta connectivity
        */
        MetaScheme meta = MetaSchemeFactory.getMetaScheme(scheme1);
        out.println(meta);
        assertEqual(5, meta.getLength(),
                    "5", meta.getLength());
        Sequence<DataColumn> columns = meta.getColumns();

        assertEqual("Customer", columns.get(0).getName(),
                    "Customer", columns.get(0).getName());
        assertEqual("Number", columns.get(1).getName(),
                    "Number", columns.get(1).getName());
        assertEqual("Name", columns.get(2).getName(),
                    "Name", columns.get(2).getName());
        assertEqual("Address", columns.get(3).getName(),
                    "Address", columns.get(3).getName());
        assertEqual("Product", columns.get(4).getName(),
                    "Product", columns.get(4).getName());


    }
}
