package test;

import static org.jtiger.framework.FixturesRunnerConfigFactory.newFixturesRunnerConfig;
import static org.jtiger.framework.ReadOnlyArrayFactory.newReadOnlyArray;
import static org.jtiger.framework.FixturesRunnerFactory.newFixturesRunner;
import org.jtiger.framework.*;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertNull;

import org.kaiwitte.workbench.data.DataSchemeFactory;
import org.kaiwitte.workbench.data.DataSchemeFactoryImpl;
import org.kaiwitte.workbench.data.DataScheme;
import org.kaiwitte.workbench.data.DataColumn;

import java.net.URI;
import java.net.URISyntaxException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import static java.lang.System.*;

import net.tmorris.adt.sequence.*;
import net.tmorris.adt.sequence.Sequence;

@Fixture("schemeFactory")
@Category("schemeFactory")
public class SchemeFactoryTest {
    @Test("scheme creation 1")
    public void createScheme() throws URISyntaxException, IOException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemes.xml");
        Sequence<DataScheme> schemes = factory.createDataScheme(in);
        assertEqual(1, schemes.getSize());
        DataScheme scheme = schemes.get(0);

        assertEqual(2, scheme.getLength());
        assertEqual(2, scheme.getColumns().getSize());
        assertEqual("incoming", scheme.getName());

        DataColumn column1 = scheme.getColumns().get(0);
        DataColumn column2 = scheme.getColumns().get(1);

        assertEqual(Integer.class, column1.getType());
        assertFalse(column1.isRequiredForUpdate());
        assertEqual("Nummer", column1.getName());
        assertNull(column1.getAssociatedDataScheme());
        assertNull(column1.getNameInAssociatedDataScheme());

        assertEqual(String.class, column2.getType());
        assertFalse(column2.isRequiredForUpdate());
        assertEqual("Bezeichnung", column2.getName());
        assertNull(column2.getAssociatedDataScheme());
        assertNull(column2.getNameInAssociatedDataScheme());
    }

    @Test("connected scheme creation 1")
    public void createConnectedSchemes() throws URISyntaxException, IOException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemesconnected.xml");
        Sequence<DataScheme> schemes = factory.createDataScheme(in);
        assertEqual(2, schemes.getSize());
        DataScheme scheme1 = schemes.get(0);
        DataScheme scheme2 = schemes.get(1);

        assertEqual(2, scheme1.getLength());
        assertEqual(2, scheme1.getColumns().getSize());
        assertEqual("sells", scheme1.getName());

        assertEqual(3, scheme2.getLength());
        assertEqual(3, scheme2.getColumns().getSize());
        assertEqual("sells", scheme1.getName());

        DataColumn column11 = scheme1.getColumns().get(0);
        DataColumn column12 = scheme1.getColumns().get(0);

        assertEqual(Integer.class, column11.getType());
        assertFalse(column11.isRequiredForUpdate());
        assertEqual("Customer", column11.getName());
        assertEqual(scheme2, column11.getAssociatedDataScheme());
        assertEqual("Number", column11.getNameInAssociatedDataScheme());
    }

    @Test("nulltest")
    @ExpectException(NullPointerException.class)
    public void testNull() throws IOException {
        new DataSchemeFactoryImpl().createDataScheme(null);
    }
}
