package test;

import org.jtiger.framework.Fixture;
import org.jtiger.framework.Category;
import org.jtiger.framework.Test;

import static org.jtiger.framework.ReadOnlyArrayFactory.newReadOnlyArray;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertFalse;

import org.kaiwitte.workbench.data.*;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModel;
import org.kaiwitte.workbench.ui.model.MetaSchemeTableModelFactory;

import java.net.URISyntaxException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;

import static java.lang.System.*;

import net.tmorris.adt.sequence.Sequence;

@Fixture("table model")
@Category("table model")
public class SchemeTableModelTest {
    @Test("proper acceptance of records")
    public void addRecord() throws IllegalAccessException, SQLException,
            InstantiationException, ClassNotFoundException, IOException {
        /*
         * Enter data to table.
         */
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemes02.xml");
        Sequence<DataScheme> schemes = factory.createDataScheme(in);

        DataManagerTest.deleteTableContent(schemes.get(0).getName());

        DataManagerFactory dataManagerFactory =
            new DataManagerFactoryImpl("com.mysql.jdbc.Driver",
                                       "jdbc:mysql://localhost/test?user=&password=");
        DataManager data = dataManagerFactory.getDataManager(
                schemes.get(0));

        Record newRecord = new MutableRecordImpl(schemes.get(0), "Kiel", 7);

        data.insert(newRecord);

        /*
        * Retrieve data from table.
        */
        Condition c = ConditionFactory.columnEqual(
            schemes.get(0).getColumns().get(0), "Kiel");

        Sequence<Record> results = data.find(c);

        /*
        * Put data into table model
        */
//        MetaSchemeTableModel table = new MetaSchemeTableModelImpl(dataManagerFactory, 
//                                                          new MetaSchemeImpl(schemes.get(0)));
//        
//        table.addRecord(results[0]);
//        
//        assertEqual(results.get(0).getID(), table.getValueAt(0, 0));
//        assertEqual("Kiel", table.getValueAt(0, 1),
//                    "Kiel", table.getValueAt(0, 1));
//        assertEqual(7, table.getValueAt(0, 2));
    }

    @Test("connected scheme table model")
    public void connectedSchemes() throws URISyntaxException, IOException,
        IllegalAccessException, SQLException, InstantiationException,
        ClassNotFoundException {
        DataSchemeFactory factory = new DataSchemeFactoryImpl();
        InputStream in = getClass().getResourceAsStream("/schemesconnected.xml");
        Sequence<DataScheme> schemes = factory.createDataScheme(in);
        assertEqual(2, schemes.getSize());
        DataScheme scheme1 = schemes.get(0);
        DataScheme scheme2 = schemes.get(1);
        MetaScheme metaScheme = MetaSchemeFactory.getMetaScheme(schemes.get(0));

        assertEqual(2, scheme1.getLength());
        assertEqual(2, scheme1.getColumns().getSize());
        assertEqual("sells", scheme1.getName());

        assertEqual(3, scheme2.getLength());
        assertEqual(3, scheme2.getColumns().getSize());
        assertEqual("customers", scheme2.getName());

        DataColumn column11 = scheme1.getColumns().get(0);
        DataColumn column12 = scheme1.getColumns().get(1);

        assertEqual(Integer.class, column11.getType());
        assertFalse(column11.isRequiredForUpdate());
        assertEqual("Customer", column11.getName());
        assertEqual(scheme2, column11.getAssociatedDataScheme());
        assertEqual("Number", column11.getNameInAssociatedDataScheme());


        /*
        * store data in db
        */
        MutableRecord record1 = new MutableRecordImpl(schemes.get(0), (Object) 23, "Netzteil");
        MutableRecord record2 = new MutableRecordImpl(schemes.get(0), (Object) 23, "Kai",
                                                      "Lutherstr.");

        MetaRecord metaRecord = new MetaRecordImpl(
            new Record[]{record1, record2});

        MetaDataManagerFactory dataManagerFactory =
            new MetaDataManagerFactoryImpl("com.mysql.jdbc.Driver",
                                           "jdbc:mysql://localhost/test?user=&password=");
        MetaDataManager data1 = dataManagerFactory.getMetaDataManager(
                metaScheme);

        data1.insert(metaRecord);


        /*
         * Retrieve data from table.
         */
        Condition c = ConditionFactory.columnEqual(
            scheme1.getColumns().get(1), "Netzteil");

        Sequence<MetaRecord> results = data1.find(c);

        /*
         * Put data into table model
         */
        MetaSchemeTableModel table =
            MetaSchemeTableModelFactory.getMetaSchemeTableModel(MetaSchemeFactory.getMetaScheme(schemes.get(0)));
        out.println(MetaSchemeFactory.getMetaScheme(schemes.get(0)));
        assertEqual(Integer.class, table.getColumnClass(0),
                    Integer.class, table.getColumnClass(0));
        assertEqual(Integer.class, table.getColumnClass(1),
                    Integer.class, table.getColumnClass(1));
        assertEqual(String.class, table.getColumnClass(2),
                    String.class, table.getColumnClass(2));
        assertEqual(String.class, table.getColumnClass(3),
                    String.class, table.getColumnClass(3));
        assertEqual(String.class, table.getColumnClass(3),
                    String.class, table.getColumnClass(3));

        table.addRecord(results.get(0));

        //assertEqual(results.get(0).getID(), table.getValueAt(0, 0));
        assertEqual(23, table.getValueAt(0, 1),
                    23, table.getValueAt(0, 1));
        assertEqual("Kai", table.getValueAt(0, 2),
                    "Kai", table.getValueAt(0, 2));
        assertEqual("Lutherstr.", table.getValueAt(0, 3),
                    "Lutherstr.", table.getValueAt(0, 3));
        assertEqual("Netzteil", table.getValueAt(0, 4),
                    "Netzteil", table.getValueAt(0, 4));
    }
}
